# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen and Greg Hazel

from __future__ import division
from __future__ import generators

import os
import sys
import struct
import cPickle
import logging
from array import array
from BitTorrent.translation import _

from BitTorrent.hash import sha
from BitTorrent.obsoletepythonsupport import set
from BitTorrent.sparse_set import SparseSet
from BitTorrent.bitfield import Bitfield
from BitTorrent.defer import Deferred
from BitTorrent.yielddefer import launch_coroutine, _wrap_task
from BitTorrent import BTFailure

NO_PLACE = -1

ALLOCATED = -1
UNALLOCATED = -2
FASTRESUME_PARTIAL = -3

global_logger = logging.getLogger('StorageWrapper')
#global_logger.setLevel(logging.DEBUG)
#global_logger.addHandler(logging.StreamHandler(sys.stdout))


class DataPig(object):
    def __init__(self, read, add_task):
        self.add_task = add_task
        self.read = read
        self.failed_pieces = {}
        self.download_history = {}

    def got_piece(self, index, begin, length, source):
        if index in self.failed_pieces:
            df = launch_coroutine(_wrap_task(self.add_task),
                                  self._got_piece,
                                  index, begin, length, source)
            return df
        self.download_history.setdefault(index, {})
        self.download_history[index][begin] = source

    def _got_piece(self, index, begin, piece, source):
        df = self.read(index, len(piece), offset=begin)
        yield df
        data = df.getResult()
        if data != piece:
            if (index in self.download_history and
                begin in self.download_history[index]):
                d = self.download_history[index][begin]
                self.failed_pieces[index].add(d)
        self.download_history.setdefault(index, {})
        self.download_history[index][begin] = source

    def finished_piece(self, index):
        for d in self.download_history[index].itervalues():
            d.good(index)
        del self.download_history[index]
        if index in self.failed_pieces:
            for d in self.failed_pieces[index]:
                d.bad(index)
            del self.failed_pieces[index]        

    def failed_piece(self, index):
        self.failed_pieces[index] = set()
        allsenders = {}
        for d in self.download_history[index].itervalues():
            allsenders[d] = None
        if len(allsenders) == 1:
            culprit = allsenders.keys()[0]
            culprit.bad(index, bump = True)
            del self.failed_pieces[index] # found the culprit already
        
current_version = 2
resume_prefix = 'BitTorrent resume state file, version '
version_string = resume_prefix + str(current_version)

class StorageWrapper(object):

    READ_AHEAD_BUFFER_SIZE = 2**22 # 4mB

    def __init__(self, storage, config, hashes, piece_size,
                 statusfunc, doneflag, data_flunked,
                 infohash, # needed for partials
                 errorfunc, working_path, destination_path, resumefile,
                 add_task, external_add_task):
        assert len(hashes) > 0
        assert piece_size > 0
        self.initialized = False
        self.numpieces = len(hashes)
        self.infohash = infohash
        self.add_task = add_task
        self.external_add_task = external_add_task
        self.storage = storage
        self.config = config
        self.doneflag = doneflag
        self.hashes = hashes
        self.piece_size = piece_size
        self.data_flunked = data_flunked
        self.errorfunc = errorfunc
        self.statusfunc = statusfunc
        self.total_length = storage.get_total_length()
        # a brief explanation about the mildly confusing amount_ variables:
        #   amount_left: total_length - fully_written_pieces
        #   amount_inactive: amount_left - requests_pending_on_network
        #   amount_left_with_partials (only correct during startup): amount_left + blocks_written
        self.amount_left = self.total_length
        self.amount_inactive = self.total_length
        if self.total_length <= piece_size * (self.numpieces - 1):
            raise BTFailure(_("bad data in responsefile - total too small"))
        if self.total_length > piece_size * self.numpieces:
            raise BTFailure(_("bad data in responsefile - total too big"))

        # If chunks have been requested then inactive_requests has a list
        # of the unrequested chunks. Otherwise the piece is not in the dict.
        self.inactive_requests = {}

        # If chunks have been requested then active_requests has a list
        # of the unrequested chunks which are pending on the network.
        # Otherwise the piece is not in the dict.
        self.active_requests = {}
        
        read = lambda index, length, offset : self._storage_read(self.places[index],
                                                                 length, offset=offset)
        self.datapig = DataPig(read, self.add_task)
        
        self.full_pieces = SparseSet()
        # a index => df dict for locking pieces
        self.blocking_pieces = {}
        self.endgame = False
        self.have = Bitfield(self.numpieces)
        self.have_set = SparseSet()
        self.checked_pieces = SparseSet()
        self.fastresume = False
        self.fastresume_dirty = False

        self._pieces_in_buf = []
        self._piece_buf = None

        self.partial_mark = None

        if self.numpieces < 32768:
            self.typecode = 'h'
        else:
            self.typecode = 'l'

        self.places = array(self.typecode, [NO_PLACE] * self.numpieces)
        check_hashes = self.config['check_hashes']

        self.done_checking_df = Deferred()
        self.lastlen = self._piecelen(self.numpieces - 1)

        global_logger.debug("Loading fastresume...")
        if not check_hashes:
            self.rplaces = array(self.typecode, range(self.numpieces))
            self._initialized(True)
        else:
            try:
                result = self.read_fastresume(resumefile, working_path,
                                              destination_path)
                # if resume file doesn't apply to this destination or
                # working path then start over.
                if not result:
                    self.rplaces = array(self.typecode, [UNALLOCATED] * self.numpieces)
                    # full hashcheck
                    df = self.hashcheck_pieces()
                    df.addCallback(self._initialized)
            except:
                if resumefile is not None:
                    global_logger.warning("Failed to read fastresume",
                                          exc_info=sys.exc_info())
                self.rplaces = array(self.typecode, [UNALLOCATED] * self.numpieces)
                # full hashcheck
                df = self.hashcheck_pieces()
                df.addCallback(self._initialized)
                
    def _initialized(self, v):
        self._pieces_in_buf = []
        self._piece_buf = None
        self.initialized = v
        global_logger.debug('Initialized')
        self.done_checking_df.callback(v)
                
    ## fastresume
    ############################################################################
    def read_fastresume(self, f, working_path, destination_path):
        version_line = f.readline().strip()
        try:
            resume_version = version_line.split(resume_prefix)[1]
        except Exception, e:
            raise BTFailure(_("Unsupported fastresume file format, "
                              "probably corrupted: %s on (%s)") %
                            (e, repr(version_line)))
        global_logger.debug('Reading fastresume v' + resume_version)
        if resume_version == '1':
            return self._read_fastresume_v1(f, working_path, destination_path)
        elif resume_version == '2':
            return self._read_fastresume_v2(f, working_path, destination_path)
        else:
            raise BTFailure(_("Unsupported fastresume file format, "
                              "maybe from another client version?"))
        
    def _read_fastresume_v1(self, f ): #, working_path, destination_path):
        # skip a bunch of lines
        amount_done = int(f.readline())
        for b, e, filename in self.storage.ranges:
            line = f.readline()

        # now for the good stuff
        r = array(self.typecode)
        r.fromfile(f, self.numpieces)

        self.rplaces = r

        df = self.checkPieces_v1()
        df.addCallback(self._initialized)

    def checkPieces_v1(self):
        df = launch_coroutine(_wrap_task(self.add_task),
                              self._checkPieces_v1)
        return df
        
    def _checkPieces_v1(self):
        partials = {}

        needs_full_hashcheck = False
        for i in xrange(self.numpieces):

            piece_len = self._piecelen(i)

            t = self.rplaces[i]
            if t >= 0:
                self._markgot(t, i)
            elif t in (ALLOCATED, UNALLOCATED):
                pass
            elif t == FASTRESUME_PARTIAL:
                df = self._storage_read(i, piece_len)
                yield df
                try:
                    data = df.getResult()
                except:
                    global_logger.error(_("Bad fastresume info (truncation at piece %d)") % i)
                    needs_full_hashcheck = True
                    i -= 1
                    break                    
                self._check_partial(i, partials, data)
                self.rplaces[i] = ALLOCATED

                # we're shutting down, abort.
                if self.doneflag.isSet():
                    yield False
            else:
                global_logger.error(_("Bad fastresume info (illegal value at piece %d)") % i)
                needs_full_hashcheck = True
                i -= 1
                break

        if needs_full_hashcheck:
            df = self.hashcheck_pieces(i)
            yield df
            r = df.getResult()
            if r == False:
                yield False

        self._realize_partials(partials)            
        yield True

    def _read_fastresume_v2(self, f, working_path, destination_path):
        # The working and destination paths are "save_as" paths meaning
        # that they refer to the entire path for a single-file torrent and the
        # name of the directory containing the files for a batch torrent.

        # Path read from resume should either reside in/at the
        # working_path or the destination_path.

        d = cPickle.loads(f.read())        

        try:
            snapshot = d['snapshot']
            work_or_dest = 0
            for filename, s in snapshot.iteritems():
                # all files should reside in either the working path or the
                # destination path.  For batch torrents, the file may have a
                # relative path so compare common path.
                if len(snapshot) > 1:  # batch torrent.
                    commonw = os.path.commonprefix([filename,working_path])
                    commond = os.path.commonprefix([filename,destination_path])
                else:
                    commonw = commond = filename

                # first file determines whether all are in work or dest path.
                if work_or_dest == 0:
                    if commonw == working_path:
                        work_or_dest = -1
                    elif commond == destination_path:
                        work_or_dest = 1
                    else:
                        return False
                elif work_or_dest == -1 and commonw != working_path:
                    return False
                elif work_or_dest == 1 and commond != dest_path:
                    return False
                    
                # this could be a lot smarter, like punching holes in the
                # ranges on failed files in a batch torrent.
                if not os.path.exists(filename):
                    raise ValueError("No such file or directory: %s" % filename)
                if os.path.getsize(filename) < s['size']:
                    raise ValueError("File sizes do not match.")
                if os.path.getmtime(filename) < (s['mtime'] - 5):
                    raise ValueError("File modification times do not match.")
                
            self.places = array(self.typecode)
            self.places.fromstring(d['places'])
            self.rplaces = array(self.typecode)
            self.rplaces.fromstring(d['rplaces'])
            self.have = d['have']
            self.have_set = d['have_set']
            # We are reading the undownloaded section from the fast resume.
            # We should check whether the file exists.  If it doesn't then
            # we should not read from fastresume.
            self.storage.undownloaded = d['undownloaded']
            self.amount_left = d['amount_left']
            
            self.amount_inactive = self.amount_left
            # all unwritten partials are now inactive
            self.inactive_requests = d['unwritten_partials']

            self.amount_left_with_partials = self.amount_left

            for k, v in self.inactive_requests.iteritems():
                s = [b for b, e in v]
                if s:
                    self._make_pending(k, s)
                self.active_requests.setdefault(k, [])

            if self.amount_left_with_partials < 0:
                raise ValueError("Amount left < 0: %d" % self.amount_left_with_partials)
            if self.amount_left_with_partials > self.total_length:
                raise ValueError("Amount left > total length: %d > %d" % (self.amount_left_with_partials, self.total_length))

            self._initialized(True)
        except:
            self.have = Bitfield(self.numpieces)
            self.have_set = SparseSet()
            self.inactive_requests = {}
            self.active_requests = {}
            self.places = array(self.typecode, [NO_PLACE] * self.numpieces)
            self.rplaces = array(self.typecode, range(self.numpieces))
            raise
        
        return True
    

    def write_fastresume(self, resumefile):
        try:
            self._write_fastresume_v2(resumefile)
        except:
            import traceback
            traceback.print_exc()
            

    def _write_fastresume_v1(self, resumefile):
        if not self.initialized:
            return

        global_logger.debug('Writing fast resume: %s' % version_string)
        resumefile.write(version_string + '\n')
        # write fake junk
        resumefile.write(str(0) + '\n')
        for b, e, filename in self.storage.ranges:
            resumefile.write(str(0) + ' ' +
                             str(0) + '\n')
        
        # copy the array so as not to screw the current state of things
        rplaces = array(self.rplaces.typecode, list(self.rplaces))
        # Ow. -G
        for i in xrange(self.numpieces):
            if rplaces[i] >= 0 and not self.have[rplaces[i]]:
                rplaces[i] = FASTRESUME_PARTIAL
        rplaces.tofile(resumefile)
        self.fastresume_dirty = False

    def _write_fastresume_v2(self, resumefile):
        if not self.initialized:
            return
        
        global_logger.debug('Writing fast resume: %s' % version_string)
        resumefile.write(version_string + '\n')

        d = {}

        snapshot = {}
        for filename in self.storage.range_by_name.iterkeys():
            if not os.path.exists(filename):
                continue
            s = {}
            s['size'] = os.path.getsize(filename)
            s['mtime'] = os.path.getmtime(filename)
            snapshot[filename] = s
        d['snapshot'] = snapshot
        
        d['places'] = self.places.tostring()
        d['rplaces'] = self.rplaces.tostring()
        d['have'] = self.have
        d['have_set'] = self.have_set
        d['undownloaded'] = self.storage.undownloaded
        d['amount_left'] = self.amount_left
        # collapse inactive and active requests into unwritten partials
        unwritten_partials = {}
        for k, v in self.inactive_requests.iteritems():
            if v:
                unwritten_partials.setdefault(k, []).extend(v)
        for k, v in self.active_requests.iteritems():
            if v:
                unwritten_partials.setdefault(k, []).extend(v)
        d['unwritten_partials'] = unwritten_partials

        resumefile.write(cPickle.dumps(d))

        self.fastresume_dirty = False

    ############################################################################

    def _realize_partials(self, partials):
        self.amount_left_with_partials = self.amount_left
        for piece in partials:
            if self.places[piece] < 0:
                pos = partials[piece][0]
                self.places[piece] = pos
                self.rplaces[pos] = piece

    def _markgot(self, piece, pos):
        if self.have[piece]:
            if piece != pos:
                return
            self.rplaces[self.places[pos]] = ALLOCATED
            self.places[pos] = self.rplaces[pos] = pos
            return
        self.places[piece] = pos
        self.rplaces[pos] = piece
        self.have[piece] = True
        self.have_set.add(piece)
        plen = self._piecelen(piece)
        self.storage.downloaded(self.piece_size * piece, plen)
        self.amount_left -= plen
        self.amount_inactive -= plen
        assert piece not in self.inactive_requests

    ## hashcheck    
    ############################################################################
    def _get_data(self, i):
        if i in self._pieces_in_buf:
            p = i - self._pieces_in_buf[0]
            return buffer(self._piece_buf, p * self.piece_size, self._piecelen(i))
        df = launch_coroutine(_wrap_task(self.add_task),
                              self._get_data_gen, i)
        return df

    def _get_data_gen(self, i):
        num_pieces = int(max(1, self.READ_AHEAD_BUFFER_SIZE / self.piece_size))
        if i + num_pieces >= self.numpieces:
            size = self.total_length - (i * self.piece_size)
            num_pieces = self.numpieces - i
        else:
            size = num_pieces * self.piece_size
        self._pieces_in_buf = range(i, i + num_pieces)
        df = self._storage_read(i, size)
        yield df
        try:
            self._piece_buf = df.getResult()
        except BTFailure: # short read
            self._piece_buf = ''
        p = i - self._pieces_in_buf[0]
        yield buffer(self._piece_buf, p * self.piece_size, self._piecelen(i))
        
    def hashcheck_pieces(self, begin=0, end=None):
        df = launch_coroutine(_wrap_task(self.add_task),
                              self._hashcheck_pieces,
                              begin, end)
        return df

    def _hashcheck_pieces(self, begin=0, end=None):
        # we need a full reverse-lookup of hashes for out of order compatability
        targets = {}
        for i in xrange(self.numpieces):
            targets[self.hashes[i]] = i
        partials = {}

        if end is None:
            end = self.numpieces

        global_logger.debug('Hashcheck from %d to %d' % (begin, end))

        # TODO: make this work with more than one running at a time
        for i in xrange(begin, end):

            # we're shutting down, abort.
            if self.doneflag.isSet():
                yield False                

            piece_len = self._piecelen(i)
            #global_logger.debug( "i=%d, piece_len=%d" % (i,piece_len) )

            if not self._waspre(i, piece_len):
                # hole in the file
                continue

            r = self._get_data(i)
            if isinstance(r, Deferred):
                yield r
                data = r.getResult()
            else:
                data = r
            
            sh = sha(buffer(data, 0, self.lastlen))
            sp = sh.digest()
            sh.update(buffer(data, self.lastlen))
            s = sh.digest()
            # handle out-of-order pieces
            if s in targets and piece_len == self._piecelen(targets[s]):
                self.checked_pieces.add(targets[s])
                self._markgot(targets[s], i)
            # last piece junk. I'm not even sure this is right.
            elif (not self.have[self.numpieces - 1] and
                  sp == self.hashes[-1] and
                  (i == self.numpieces - 1 or
                   not self._waspre(self.numpieces - 1))):
                self.checked_pieces.add(self.numpieces - 1)
                self._markgot(self.numpieces - 1, i)
            else:
                self._check_partial(i, partials, data)
            self.statusfunc(fractionDone = 1 - self.amount_left /
                            self.total_length)
            
        global_logger.debug('Hashcheck from %d to %d complete.' % (begin, end))

        self._realize_partials(partials)
        self.fastresume_dirty = True
        yield True

    def hashcheck_piece(self, index, data = None):
        df = launch_coroutine(_wrap_task(self.add_task),
                              self._hashcheck_piece,
                              index, data = data)
        return df

    def _hashcheck_piece(self, index, data = None):
        if not data:
            df = self._storage_read(index, self._piecelen(index))
            yield df
            data = df.getResult()
        if sha(data).digest() != self.hashes[index]:
            yield False
        self.checked_pieces.add(index)
        yield True
    ############################################################################
    
    ## out of order compatability
    ############################################################################
    def _initalloc(self, pos, piece):
        assert self.rplaces[pos] < 0
        assert self.places[piece] == NO_PLACE
        p = self.piece_size * pos
        length = self._piecelen(pos)
        self.places[piece] = pos
        self.rplaces[pos] = piece

    def _move_piece(self, oldpos, newpos):
        assert self.rplaces[newpos] < 0
        assert self.rplaces[oldpos] >= 0
        df = self._storage_read(oldpos, self._piecelen(newpos))
        yield df
        data = df.getResult()
        df = self._storage_write(newpos, data)
        yield df
        df.getResult()
        piece = self.rplaces[oldpos]
        self.places[piece] = newpos
        self.rplaces[oldpos] = ALLOCATED
        self.rplaces[newpos] = piece
        if not self.have[piece]:
            return
        data = buffer(data, 0, self._piecelen(piece))
        if sha(data).digest() != self.hashes[piece]:
            raise BTFailure(_("data corrupted on disk - "
                              "maybe you have two copies running?"))
    ############################################################################



    def get_piece_range_for_filename(self, filename):
        begin, end = self.storage.get_byte_range_for_filename(filename)
        begin = int(begin / self.piece_size)
        end = int(end / self.piece_size)
        return begin, end

    def _waspre(self, piece, piece_len=None):
        if piece_len is None:
            piece_len = self._piecelen(piece)
        return self.storage.was_preallocated(piece * self.piece_size, piece_len)

    def _piecelen(self, piece):
        if piece < self.numpieces - 1:
            return self.piece_size
        else:
            return self.total_length - piece * self.piece_size

    def get_total_length(self):
        """Returns the total length of the torrent in bytes."""
        return self.total_length

    def get_num_pieces(self):
        """Returns the total number of pieces in this torrent."""
        return self.numpieces

    def get_amount_left(self):
        """Returns the number of bytes left to download."""
        return self.amount_left

    def do_I_have_anything(self):
        return self.amount_left < self.total_length

    def get_have_list(self):
        return self.have.tostring()

    def do_I_have(self, index):
        return self.have[index]

    def _block_piece(self, index, df):
        self.blocking_pieces[index] = df
        df.addCallback(lambda x: self.blocking_pieces.pop(index))
        return df

    def write(self, index, begin, piece, source):
        df = launch_coroutine(_wrap_task(self.add_task),
                              self._write,
                              index, begin, piece, source)
        return df

    def _write(self, index, begin, piece, source):

        if index in self.blocking_pieces:
            df = self.blocking_pieces[index]
            yield df
            df.getResult()

        if self.places[index] < 0:
            # since old versions of BT wrote out-of-order, we could
            # come across a piece which is misplaced. move it to the
            # correct place.
            if self.rplaces[index] >= 0:
                new_pos = self.rplaces[index]
                df = launch_coroutine(_wrap_task(self.add_task),
                                      self._move_piece, index, new_pos)
                yield self._block_piece(index, df)
                df.getResult()

            self._initalloc(index, index)
            
        df = self.datapig.got_piece(index, begin, piece, source)
        if df is not None:
            yield df
            df.getResult()

        df = self._storage_write(self.places[index], piece, offset=begin)
        yield df
        df.getResult()
        r = (begin, len(piece))
        self.active_requests[index].remove(r)

        hashcheck = False
        if not self.want_requests(index) and len(self.active_requests[index]) == 0:
            hashcheck = True
            df = self.hashcheck_piece(self.places[index])
            yield df
            passed = df.getResult()
            length = self._piecelen(index)
            del self.inactive_requests[index]
            del self.active_requests[index]
            self.full_pieces.remove(index)
            if passed:
                self.have[index] = True
                self.have_set.add(index)
                # In this function we perform a lookup undownloaded[filename]
                # which results in a KeyError.  --Dave HEREDAVE
                self.storage.downloaded(index * self.piece_size, length)
                self.amount_left -= length

                self.datapig.finished_piece(index)
            else: # hashcheck fail
                self.data_flunked(length, index)
                self.amount_inactive += length

                self.datapig.failed_piece(index)
        self.fastresume_dirty = True
        yield hashcheck

    def read(self, index, begin, length):
        df = launch_coroutine(_wrap_task(self.add_task),
                              self._read, index, begin, length)
        return df

    def _read(self, index, begin, length):
        if not self.have[index]:
            yield None

        if index in self.blocking_pieces:
            df = self.blocking_pieces[index]
            yield df
            df.getResult()

        if index not in self.checked_pieces:
            df = self.hashcheck_piece(self.places[index])
            yield df
            passed = df.getResult()
            if not passed:
                # TODO: this case should cause a total file hash check and
                # reconnect when done.
                raise BTFailure, _("told file complete on start-up, but piece failed hash check")
        if begin + length > self._piecelen(index):
            yield None
        df = self._storage_read(self.places[index], length, offset=begin)
        yield df
        data = df.getResult()
        yield data

    def _storage_read(self, index, amount, offset=0):
        return self.storage.read(index * self.piece_size + offset, amount)

    def _storage_write(self, index, data, offset=0):
        return self.storage.write(index * self.piece_size + offset, data)

    ## partials
    ############################################################################
    def _check_partial(self, pos, partials, data):
        index = None
        missing = False
        if self.partial_mark is None:
            self.partial_mark = ("BitTorrent - this part has not been downloaded " +
                                 "yet." + self.infohash +
                                 struct.pack('>i', self.config['download_slice_size']))
        marklen = len(self.partial_mark)+4
        for i in xrange(0, len(data) - marklen,
                        self.config['download_slice_size']):
            if data[i:i+marklen-4] == self.partial_mark:
                ind = struct.unpack('>i', data[i+marklen-4:i+marklen])[0]
                if index is None:
                    index = ind
                    parts = []
                if ind >= self.numpieces or ind != index:
                    return
                parts.append(i)
            else:
                missing = True
        if index is not None and missing:
            i += self.config['download_slice_size']
            if i < len(data):
                parts.append(i)
            partials[index] = (pos, parts)

    def _make_pending(self, index, parts):
        length = self._piecelen(index)
        x = 0
        request_size = self.config['download_slice_size']
        for x in xrange(0, length, request_size):
            partlen = min(request_size, length - x)
            if x not in parts:
                self.amount_left_with_partials -= partlen
    ############################################################################


    ## request manager stuff
    ############################################################################
    def want_requests(self, index):
        # blah, this is dumb.
        if self.have[index]:
            return False
        # if all requests are pending, we'll have a blank list
        if (index in self.inactive_requests and
            len(self.inactive_requests[index]) == 0):
            return False
        return True

    def iter_want(self):
        for index in self.have_set.iterneg(0, self.numpieces):
            # if all requests are pending, we'll have a blank list
            if (index in self.inactive_requests and
                len(self.inactive_requests[index]) == 0):
                continue
            yield index

    def new_request(self, index, full=False):
        # returns (begin, length)
        if index not in self.inactive_requests:
            self._make_inactive(index)
        rs = self.inactive_requests[index]
        if full:
            s = SparseSet()
            while rs:
                r = rs.pop()
                s.add(r[0], r[0] + r[1])
            r = s.largest_range()
            s.remove(*r)
            for b, e in s.iterrange():
                l = e - b
                rs.extend(self._break_up(b, l))
        else:
            # why min? do we want all the blocks in order?
            r = min(rs)
            rs.remove(r)
        if len(rs) == 0:
            self.full_pieces.add(index)
        self.amount_inactive -= r[1]
        assert self.amount_inactive >= 0, 'Amount inactive: %d' % self.amount_inactive
        if self.amount_inactive == 0:
            self.endgame = True
        self.active_requests[index].append(r)
        return r

    def _break_up(self, begin, length):
        l = []
        x = 0
        request_size = self.config['download_slice_size']
        while x + request_size < length:
            l.append((begin + x, request_size))
            x += request_size
        l.append((begin + x, length - x))
        return l

    def _make_inactive(self, index):
        length = self._piecelen(index)
        self.inactive_requests[index] = self._break_up(0, length)
        self.active_requests[index] = []

    def request_lost(self, index, begin, length):
        if len(self.inactive_requests[index]) == 0:
            self.full_pieces.remove(index)
        self.amount_inactive += length
        r = (begin, length)
        self.active_requests[index].remove(r)
        l = self._break_up(begin, length)
        for r in l:
            self.inactive_requests[index].append(r)
    ############################################################################



import wx
from BitTorrent.translation import _
from BitTorrent.GUI_wx import BTDialog, text_wrappable, ElectroStaticBitmap

ID_COMBOBOX = wx.NewId()
ID_BROWSE = wx.NewId()

class OpenDialog(BTDialog):
    
    def __init__(self, parent, bitmap, browse, history, *a, **kw):
        BTDialog.__init__(self, parent, *a, **kw)

        itemDialog1 = self
        self.browse_func = browse

        itemFlexGridSizer2 = wx.FlexGridSizer(3, 1, 3, 0)
        itemFlexGridSizer2.AddGrowableCol(0)
        itemDialog1.SetSizer(itemFlexGridSizer2)

        itemFlexGridSizer3 = wx.FlexGridSizer(2, 2, 21, 0)
        itemFlexGridSizer3.AddGrowableCol(1)
        itemFlexGridSizer2.Add(itemFlexGridSizer3, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        itemStaticBitmap4Bitmap = bitmap
        #itemStaticBitmap4 = wx.StaticBitmap(itemDialog1, wx.ID_STATIC, itemStaticBitmap4Bitmap)
        itemStaticBitmap4 = ElectroStaticBitmap(itemDialog1, itemStaticBitmap4Bitmap)
        itemFlexGridSizer3.Add(itemStaticBitmap4, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        itemStaticText5 = wx.StaticText( itemDialog1, wx.ID_STATIC, _("Enter the URL or path to a torrent file on the Internet, your computer, or your network that you want to add."), wx.DefaultPosition, wx.DefaultSize, 0 )
        if text_wrappable: itemStaticText5.Wrap(286)
        itemFlexGridSizer3.Add(itemStaticText5, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.ADJUST_MINSIZE, 7)

        itemStaticText6 = wx.StaticText( itemDialog1, wx.ID_STATIC, _("Open:"), wx.DefaultPosition, wx.DefaultSize, 0 )
        itemFlexGridSizer3.Add(itemStaticText6, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.ADJUST_MINSIZE, 5)

        choiceboxStrings = history
        self.choicebox = wx.ComboBox( itemDialog1, ID_COMBOBOX, choices=choiceboxStrings, size=(267, -1), style=wx.CB_DROPDOWN|wx.TE_PROCESS_ENTER )
        self.choicebox.Bind(wx.EVT_TEXT, self.OnText)
        self.choicebox.Bind(wx.EVT_COMBOBOX, self.OnComboBox)
        self.choicebox.Bind(wx.EVT_TEXT_ENTER, self.OnTextEnter)
        itemFlexGridSizer3.Add(self.choicebox, 1, wx.ALIGN_CENTER_HORIZONTAL|wx.GROW|wx.ALL, 5)

        itemBoxSizer8 = wx.BoxSizer(wx.HORIZONTAL)
        itemFlexGridSizer2.Add(itemBoxSizer8, 0, wx.ALIGN_RIGHT|wx.ALIGN_BOTTOM|wx.TOP|wx.BOTTOM, 1)

        itemFlexGridSizer9 = wx.FlexGridSizer(2, 3, 0, 2)
        itemFlexGridSizer9.AddGrowableRow(0)
        itemBoxSizer8.Add(itemFlexGridSizer9, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 7)

        itemBoxSizer10 = wx.BoxSizer(wx.HORIZONTAL)
        itemFlexGridSizer9.Add(itemBoxSizer10, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 2)

        self.okbutton = wx.Button( itemDialog1, wx.ID_OK, _("&OK"), wx.DefaultPosition, wx.DefaultSize, 0 )
        itemBoxSizer10.Add(self.okbutton, 0, wx.GROW|wx.ALL|wx.SHAPED, 0)

        itemBoxSizer12 = wx.BoxSizer(wx.HORIZONTAL)
        itemFlexGridSizer9.Add(itemBoxSizer12, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 2)

        itemButton13 = wx.Button( itemDialog1, wx.ID_CANCEL, _("&Cancel"), wx.DefaultPosition, wx.DefaultSize, 0 )
        itemBoxSizer12.Add(itemButton13, 0, wx.GROW|wx.ALL|wx.SHAPED, 0)

        itemBoxSizer14 = wx.BoxSizer(wx.HORIZONTAL)
        itemFlexGridSizer9.Add(itemBoxSizer14, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 2)

        itemButton15 = wx.Button( itemDialog1, ID_BROWSE, _("&Browse"), wx.DefaultPosition, wx.DefaultSize, 0 )
        itemButton15.Bind(wx.EVT_BUTTON, self.browse)
        itemBoxSizer14.Add(itemButton15, 0, wx.GROW|wx.ALL|wx.SHAPED, 0)

        self.okbutton.Disable()
        self.Fit()

    def _OnChange(self, v):
        self.okbutton.Enable(bool(v))

    def OnText(self, event):
        self._OnChange(self.GetValue())

    def OnComboBox(self, event):
        wx.CallAfter(self._OnChange, event.GetString())

    def OnTextEnter(self, event):
        v = self.GetValue()
        if v:
            self.EndModal(wx.ID_OK)

    def GetValue(self):
        return self.choicebox.GetValue()

    def browse(self, event):
        self.EndModal(wx.ID_CANCEL)
        self.browse_func()



# Zeroconf discovery of other BT clients on the local network.
#
# by Greg Hazel

import sys
import random
import socket
import logging
import Zeroconf
from BitTorrent.HostIP import get_host_ip

discovery_logger = logging.getLogger('LocalDiscovery')
discovery_logger.setLevel(logging.DEBUG)
#discovery_logger.addHandler(logging.StreamHandler(sys.stdout))

class LocalDiscovery(object):

    def __init__(self, rawserver, port, got_peer):
        self.rawserver = rawserver
        self.port = port
        self.got_peer = got_peer
        self.server = Zeroconf.Zeroconf()

    def announce(self, infohash, peerid):
        discovery_logger.info("announcing: %s", infohash)
        service_name = "_BitTorrent-%s._tcp.local." % infohash
        
        # do I need to keep the browser around?
        browser = Zeroconf.ServiceBrowser(self.server, service_name, self)

        addr = socket.inet_aton(get_host_ip())
        service = Zeroconf.ServiceInfo(service_name,
                                       peerid + "." + service_name,
                                       address = addr,
                                       port = self.port,
                                       weight = 0, priority=0,
                                       properties = {}
                                      )
        self.server.registerService(service)

    def addService(self, server, type, name):
        discovery_logger.info("Service %s added", repr(name))
        # Request more information about the service
        info = server.getServiceInfo(type, name)
        if info and info.address is not None:
            host = socket.inet_ntoa(info.address)
            try:
                port = int(info.port)
            except:
                discovery_logger.exception("Invalid Service (port not an int): %s" + repr(info.__dict__))
                return
                
            addr = (host, port)
            if addr == (get_host_ip(), self.port):
                # talking to self
                return

            infohash = name.split("_BitTorrent-")[1][:-len("._tcp.local.")]

            discovery_logger.info("Got peer: %s:%d %s", host, port, infohash)

            # BUG: BitTorrent is so broken!
            t = random.random() * 3

            self.rawserver.external_add_task(t, self._got_peer, addr, infohash)

    def removeService(self, server, type, name):
        discovery_logger.info("Service %s removed", repr(name))

    def _got_peer(self, addr, infohash):
        if self.got_peer:
            self.got_peer(addr, infohash)
            
    def stop(self):
        self.port = None
        self.got_peer = None
        self.server.close()

if __name__ == '__main__':
    import string
    import threading
    from BitTorrent.RawServer_twisted import RawServer

    config = {'max_incomplete': 10,
              'max_upload_rate': 350000,
              'bind': '',
              'close_with_rst': False,
              'socket_timeout': 3000}

    event = threading.Event()
    rawserver = RawServer(config=config, noisy=True)

    rawserver.install_sigint_handler()    

    def run_task_and_exit():
        l = LocalDiscovery(rawserver, 6881, lambda *a:sys.stdout.write("GOT: %s\n" % str(a)))
        l.announce("63f27f5023d7e49840ce89fc1ff988336c514b64", ''.join(random.sample(string.letters, 5)))
    
    rawserver.add_task(0, run_task_and_exit)

    rawserver.listen_forever(event)    
    



#!/usr/bin/env python

# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Original version written by John Hoffman, heavily modified for different
# multitorrent architecture by Uoti Urpala (over 40% shorter than original),
# ported to new MultiTorrent (circa 4.20) by David Harrison.

from __future__ import division
from BitTorrent.translation import _

import os
from cStringIO import StringIO
from traceback import print_exc
import logging
from BitTorrent import configfile
from BitTorrent.parsedir import parsedir
from BitTorrent.MultiTorrent import MultiTorrent, Feedback
from BitTorrent.ConvertedMetainfo import ConvertedMetainfo
from BitTorrent import BTFailure, UserFailure
from BitTorrent.RawServer_twisted import RawServer
from BitTorrent.yielddefer import launch_coroutine, _wrap_task
from BitTorrent.ConvertedMetainfo import ConvertedMetainfo
from BitTorrent.defer import DeferredEvent
from time import time


#class LaunchMany(Feedback):
class LaunchMany(object):

    def __init__(self, config, display, configfile_key):
      """Starts torrents for all .torrent files in a directory tree.

         All errors are logged using Python logging to 'configfile_key' logger.

         @param config: Preferences object storing config.
         @param display: output function for stats.
      """
      # 4.4.x version of LaunchMany output exceptions to a displayer.
      # This version only outputs stats to the displayer.  We do not use
      # the logger to output stats so that a caller-provided object
      # can provide stats formatting as opposed to using the
      # logger Formatter, which is specific to exceptions, warnings, and
      # info messages.
      try:
        self.logger = logging.getLogger(configfile_key)
        
        self.multitorrent = None
        self.rawserver = None
        self.config = config
        self.configfile_key = configfile_key
        self.display = display

        self.torrent_dir = config['torrent_dir']

        # Ex: torrent_cache = infohash ->
        #   {path:'/a/c.torrent',file:'c.torrent',length:90911,name:'Sea'}
        self.torrent_cache = {}

        # maps path -> [(modification time, size), infohash]
        self.file_cache = {}

        # used as set containing paths of files that do not have separate
        # entries in torrent_cache either because torrent_cache already
        # contains the torrent or because the torrent file is corrupt.
        self.blocked_files = {}

        #self.torrent_list = [] 
        #self.downloads = {}

        self.hashcheck_queue = []
        self.hashcheck_store = {}
        self.hashcheck_current = None
                         
        self.core_doneflag = DeferredEvent()
        rawserver_doneflag = DeferredEvent() 
        self.rawserver = RawServer(self.config)
        try:
   
            # set up shut-down procedure before we begin doing things that
            # can throw exceptions.
            def shutdown():
                print "SHUTDOWNSHUTDOWNSHUTDOWN"
                self.logger.critical(_("shutting down"))
                for infohash in self.multitorrent.get_torrents():
                    self.logger.info(_('dropped "%s"') %
                                    self.torrent_cache[infohash]['path'])
                #    torrent = self.downloads[infohash]
                #    if torrent is not None:
                #        torrent.shutdown()
                if self.multitorrent:
                    df = self.multitorrent.shutdown()
                    set_flag = lambda *a : rawserver_doneflag.set()
                    df.addCallbacks(set_flag, set_flag)
                else:
                    rawserver_doneflag.set()
                
            # It is safe to addCallback here, because there is only one thread,
            # but even if the code were multi-threaded, core_doneflag has not
            # been passed to anyone.  There is no chance of a race condition
            # between the DeferredEvent's callback and addCallback.
            self.core_doneflag.addCallback(
                lambda r: self.rawserver.external_add_task(0, shutdown))
   
            self.rawserver.install_sigint_handler(self.core_doneflag)
    
            data_dir = config['data_dir']
            print "Creating MultiTorrent"
            self.multitorrent = MultiTorrent(config, self.rawserver, data_dir)
    
            self.rawserver.add_task(0, self.scan)
            self.rawserver.add_task(0, self.stats)
            self.rawserver.add_task(1, self.check_hashcheck_queue)
            
            try:
                import signal
                def handler(signum, frame):
                    self.rawserver.external_add_task(0, self.read_config)
                signal.signal(signal.SIGHUP, handler)
            except Exception, e:
                self.logger.error(_("Could not set signal handler: ") +
                                    unicode(e.args[0]))
                self.rawserver.add_task(0,self.core_doneflag.set())
  
        except UserFailure, e:
            output.exception(unicode(e.args[0]))
            self.rawserver.add_task(0,self.core_doneflag.set())
        except:
            data = StringIO()
            print_exc(file = data)
            output.exception(data.getvalue())
            self.rawserver.add_task(0,self.core_doneflag.set())
           
        # always make sure events get processed even if only for
        # shutting down.
        print "listening forever"
        self.rawserver.listen_forever(rawserver_doneflag)
        
      except:
        data = StringIO()
        print_exc(file = data)
        output.exception(data.getvalue())

    def scan(self):
        print "LaunchMany.scan top."
        self.rawserver.add_task(self.config['parse_dir_interval'], self.scan)

        r = parsedir(self.torrent_dir, self.torrent_cache,
                     self.file_cache, self.blocked_files,
                     self.logger.error)

        print "After parsedir"
        ( self.torrent_cache, self.file_cache, self.blocked_files,
            added, removed ) = r
        for infohash, data in removed.items():
            self.logger.info(_('dropped "%s"') % data['path'])
            self.remove(infohash)
        for infohash, data in added.items():
            print "adding item:", data['path']
            self.logger.info(_('added "%s"'  ) % data['path'])
            print "after self.logger.info"
            if self.config['launch_delay'] > 0:
                self.rawserver.add_task(self.config['launch_delay'], self.add, infohash, data)
            # torrent may have been known from resume state.
            elif not self.multitorrent.torrent_known(infohash):
                self.add(infohash, data)
        print "LaunchMany.scan bottom"

    def stats(self):
        self.rawserver.add_task(self.config['display_interval'], self.stats)
        data = []
        for d in self.get_torrents():
            infohash = d.infohash
            cache = self.torrent_cache[infohash]
            if self.config['display_path']:
                name = cache['path']
            else:
                name = cache['name']
            size = cache['length']
            #d = self.downloads[infohash]
            progress = '0.0%'
            peers = 0
            seeds = 0
            seedsmsg = "S"
            dist = 0.0
            uprate = 0.0
            dnrate = 0.0
            upamt = 0
            dnamt = 0
            t = 0
            msg = ''
            #if d.state in ["created", "initializing"]:
            #    status = _("waiting for hash check")
            #else:
            stats = d.get_status()
            status = stats['activity']
            progress = '%.1f%%' % (int(stats['fractionDone']*1000)/10.0)
            if d.is_running():
                s = stats
                dist = s['numCopies']
                if d.is_seed:
                    seeds = 0 # s['numOldSeeds']
                    seedsmsg = "s"
                else:
                    if s['numSeeds'] + s['numPeers']:
                        t = stats['timeEst']
                        if t is None:
                            t = -1
                        if t == 0:  # unlikely
                            t = 0.01
                        status = _("downloading")
                    else:
                        t = -1
                        status = _("connecting to peers")
                    seeds = s['numSeeds']
                    dnrate = stats['downRate']
                peers = s['numPeers']
                uprate = stats['upRate']
                upamt = s['upTotal']
                dnamt = s['downTotal']

            data.append(( name, status, progress, peers, seeds, seedsmsg, dist,
                          uprate, dnrate, upamt, dnamt, size, t, msg ))
        stop = self.display(data)
        if stop:
            self.core_doneflag.set()

    def remove(self, infohash):
        self.torrent_list.remove(infohash)
        if self.downloads[infohash] is not None:
            self.downloads[infohash].shutdown()
        self.was_stopped(infohash)
        del self.downloads[infohash]

    def add(self, infohash, data):

        # data is a dict like
        # { path:'/a/b/c.torrent', file:'c.torrent', length:90911, name:'Sea',
        #   metainfo: <metainfo>}  Metainfo has bdecoded but not passed
        # to ConvertedMetainfo.
        self.torrent_list.append(infohash)
        self.downloads[infohash] = None
        self.hashcheck_queue.append(infohash)
        self.hashcheck_store[infohash] = ConvertedMetainfo(data['metainfo'])
        self.check_hashcheck_queue()

    def check_hashcheck_queue(self):
        if self.hashcheck_current is not None or not self.hashcheck_queue:
            return
        infohash = self.hashcheck_current = self.hashcheck_queue.pop(0)
        metainfo = self.hashcheck_store[infohash]
        del self.hashcheck_store[infohash]
        filename = self.determine_filename(infohash)
        torrent_path = self.torrent_cache[infohash]['path']
        self.start_torrent(torrent_path, metainfo, filename, filename)

    def start_torrent(self,torrent_path,metainfo,save_incomplete_as,save_as):
        assert isinstance(metainfo, ConvertedMetainfo)
        df = launch_coroutine(_wrap_task(self.rawserver.add_task),
                              self._start_torrent, metainfo,
                              save_incomplete_as, save_as)
        df.addErrback(lambda e : self.logger.error(_("DIED: "),exc_info=e))
        return df

    def _start_torrent(self, metainfo, save_incomplete_as,save_as):
        assert isinstance(metainfo, ConvertedMetainfo)
        df = self.multitorrent.create_torrent(metainfo,
                                              save_incomplete_as, save_as)
        yield df
        torrent = self.multitorrent.get_torrent(metainfo.infohash)
        if torrent.is_initialized():
           multitorrent.start_torrent(metainfo.infohash)
        #else:  ????  # this would be an error condition already reported
                      # to logger.
        check_hashcheck_queue()

    def determine_filename(self, infohash):
        x = self.torrent_cache[infohash]
        name = x['name']
        savein = self.config['save_in']
        isdir = not x['metainfo']['info'].has_key('length')
        style = self.config['saveas_style']
        if style == 4:
            torrentname   = os.path.split(x['path'][:-8])[1]
            suggestedname = name
            if torrentname == suggestedname:
                style = 1
            else:
                style = 3

        if style == 1 or style == 3:
            if savein:
                saveas = os.path.join(savein,x['file'][:-8]) # strip '.torrent'
            else:
                saveas = x['path'][:-8] # strip '.torrent'
            if style == 3 and not isdir:
                saveas = os.path.join(saveas, name)
        else:
            if savein:
                saveas = os.path.join(savein, name)
            else:
                saveas = os.path.join(os.path.split(x['path'])[0], name)
        return saveas

    def was_stopped(self, infohash):
        try:
            self.hashcheck_queue.remove(infohash)
        except:
            pass
        else:
            del self.hashcheck_store[infohash]
        if self.hashcheck_current == infohash:
            self.hashcheck_current = None
        self.check_hashcheck_queue()

    # Exceptions are now reported via loggers.<
    #def global_error(self, level, text):
    #    self.output.message(text)

    # Exceptions are now reported via loggers.
    #def exchandler(self, s):
    #    self.output.exception(s)

    def read_config(self):
        try:
            newvalues = configfile.get_config(self.config, self.configfile_key)
        except Exception, e:
            self.logger.error(_("Error reading config: ") + unicode(e.args[0]) )
            return
        self.logger.info(_("Rereading config file"))
        self.config.update(newvalues)
        # The set_option call can potentially trigger something that kills
        # the torrent (when writing this the only possibility is a change in
        # max_files_open causing an IOError while closing files), and so
        # the self.failed() callback can run during this loop.
        for option, value in newvalues.iteritems():
            self.multitorrent.set_option(option, value)
        for torrent in self.downloads.values():
            if torrent is not None:
                for option, value in newvalues.iteritems():
                    torrent.set_option(option, value)

    # rest are callbacks from torrent instances

    # DEPRECATED. 
    # 'started' is now achieved via a callback on a deferred.
    # 'failed' is now achievecd via an errback on a deferred.
    # 'exception' is now handled via the logging package.
    # Install a root log handler to catch all exceptions.
    #
    #def started(self, torrent):
    #    self.hashcheck_current = None
    #    self.check_hashcheck_queue()
    #
    #def failed(self, torrent):
    #    infohash = torrent.infohash
    #    self.was_stopped(infohash)
    #    if self.torrent_cache.has_key(infohash):
    #        self.output.message('DIED: "'+self.torrent_cache[infohash]['path']+'"')
    #
    #def exception(self, torrent, text):
    #    self.exchandler(text)



# yielddefer is an async programming mechanism with a blocking look-alike syntax
#
# from inside a generator launched with launch_coroutine:
# wait on a deferred to be called back by yielding it
# return None by simply returning
# return an exception by throwing one
# return a value by yielding a non-Deferred
#
# launch_coroutine maintains the illusion that the passed function
# (a generator) runs from beginning to end yielding when necessary
# for some job to complete and then continuing where it left off.
# 
# def f():
#    ...
#    df = some_thing_that_takes_time()
#    yield df
#    df.getResult() # Even if not expecting result.
#    ...
#    df = something_else()
#    yield df
#    result = df.getResult()
#    ...
# 
# Upon resuming from a yield point, the generator should 
# call getResult() even if no result is expected, so that
# exceptions generated while yielding are raised.
#
# by Greg Hazel

from __future__ import generators
import sys
import types
import traceback
# (half) CRUFT - remove when bittorrent uses twisted deferreds
from BitTorrent.defer import Deferred
from stackthreading import _print_traceback, _print
from twisted.internet import defer

debug = False
name_debug = False

class GenWithDeferred(object):
    if debug:
        __slots__ = ['gen', 'deferred', 'queue_task', 'stack']
    else:
        __slots__ = ['gen', 'deferred', 'queue_task']
    def __init__(self, gen, deferred, queue_task):
        self.gen = gen
        self.deferred = deferred
        self.queue_task = queue_task

        if debug:
            try:
                raise ZeroDivisionError
            except ZeroDivisionError:
                f = sys.exc_info()[2].tb_frame.f_back

            self.stack = traceback.extract_stack(f)
            # cut out GenWithDeferred() and launch_coroutine
            self.stack = self.stack[:-2]

    def cleanup(self):
        del self.gen
        del self.deferred
        del self.queue_task
        if debug:
            del self.stack

    if name_debug:
        def __getattr__(self, attr):
            if '_recall' not in attr:
                raise AttributeError(attr)
            return self._recall

        def _queue_task_chain(self, v):
            recall = getattr(self, "_recall_%s" % self.gen.gi_frame.f_code.co_name)
            self.queue_task(recall)
            return v
    else:
        def _queue_task_chain(self, v):
            self.queue_task(self._recall)
            return v

    def _recall(self):
        try:
            t = self.gen.next()
        except StopIteration:
            self.deferred.callback(None)
            self.cleanup()
        except Exception, e:

            exc_type, value, tb = sys.exc_info()

            ## Magic Traceback Hacking
            if debug:
                # interpreter shutdown
                if not sys:
                    return
                stream = sys.stderr
                # HERE.  This should really be logged or else bittorrent-
                # curses will never be able to properly output. --Dave
                _print_traceback(stream, self.stack,
                                 "generator %s" % self.gen.gi_frame.f_code.co_name, 0,
                                 exc_type, value, tb)
            else:
                #if (tb.tb_lineno != self.gen.gi_frame.f_lineno or
                #    tb.f_code.co_filename != self.gen.gi_frame.f_code.co_filename):
                #    tb = FakeTb(self.gen.gi_frame, tb)
                pass
            ## Magic Traceback Hacking
                
            self.deferred.errback((exc_type, value, tb))
            self.cleanup()
        else:
            # (half) CRUFT - remove when bittorrent uses twisted deferreds
            if not isinstance(t, Deferred) and not isinstance(t, defer.Deferred):
                self.deferred.callback(t)
                self.cleanup()
                return

            t.addCallback(self._queue_task_chain)
            t.addErrback(self._queue_task_chain)
            del t
        
class FakeTb(object):
    __slots__ = ['tb_frame', 'tb_lineno', 'tb_orig', 'tb_next']
    def __init__(self, frame, tb):
        self.tb_frame = frame
        self.tb_lineno = frame.f_lineno
        self.tb_orig = tb
        self.tb_next = tb.tb_next

def _launch_generator(queue_task, g, main_df):
    gwd = GenWithDeferred(g, main_df, queue_task)
    ## the first one is fired for you
    ##gwd._recall()
    # the first one is not fired for you, because if it errors the sys.exc_info
    # causes an unresolvable circular reference that makes the gwd.deferred never
    # be deleted.
    gwd._queue_task_chain(None)

def launch_coroutine(queue_task, f, *args, **kwargs):
    main_df = Deferred()
    try:
        g = f(*args, **kwargs)
    except Exception, e:
        if debug:
            traceback.print_exc()
        main_df.errback(sys.exc_info())
    else:        
        if isinstance(g, types.GeneratorType):
            _launch_generator(queue_task, g, main_df)
        else:
            # we got a non-generator, just callback with the return value
            main_df.callback(g)
    return main_df

def wrap_task(add_task):
    return lambda _f, *args, **kwargs : add_task(0, _f, *args, **kwargs)
_wrap_task = wrap_task   



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# GetTorrent -- abstraction which can get a .torrent file from multiple
# sources: local file, url, etc.

# written by Matt Chisholm

import os
import re
from BitTorrent import zurllib
from BitTorrent.translation import _

from bencode import bdecode, bencode
from BitTorrent.platform import get_cache_dir
from BitTorrent.ConvertedMetainfo import ConvertedMetainfo


urlpat = re.compile('^\w+://')
urlpat_torrent = re.compile('^torrent://')
urlpat_bittorrent = re.compile('^bittorrent://')

class GetTorrentException(Exception):
    pass

class UnknownArgument(GetTorrentException):
    pass

class URLException(GetTorrentException):
    pass

class FileException(GetTorrentException):
    pass

class MetainfoException(GetTorrentException):
    pass


def get_quietly(arg):
    """Wrapper for GetTorrent.get() which works around an IE bug.  If
    there's an error opening a file from the IE cache, act like we
    simply didn't get a file (because we didn't).  This happens when
    IE passes us a path to a file in its cache that has already been
    deleted because it came from a website which set Pragma:No-Cache
    on it.
    """
    try:
        data = get(arg)
    except FileException, e:
        cache = get_cache_dir() 
        if (cache is not None) and (cache in unicode(e.args[0])):
            data = None
        else:
            raise

    return data


def _get(arg):
    """Obtains the contents of the .torrent metainfo file either from
       the local filesystem or from a remote server. 'arg' is either
       a filename or an URL.

       Returns data, the raw contents of the .torrent file. Any
       exception raised while obtaining the .torrent file or parsing
       its contents is caught and wrapped in one of the following
       errors: GetTorrent.URLException, GetTorrent.FileException,
       GetTorrent.MetainfoException, or GetTorrent.UnknownArgument.
       (All have the base class GetTorrent.GetTorrentException)
       """
    data = None
    arg_stripped = arg.strip()
    if os.access(arg, os.F_OK):
        data = get_file(arg)
    elif urlpat.match(arg_stripped):
        data = get_url(arg_stripped)
    else:
        raise UnknownArgument(_('Could not read "%s", it is neither a file nor a URL.') % arg)

    return data


def get(arg):
    """Obtains the contents of the .torrent metainfo file either from
       the local filesystem or from a remote server. 'arg' is either
       a filename or an URL.

       Returns a ConvertedMetainfo object which is the parsed metainfo
       from the contents of the .torrent file.  Any exception raised
       while obtaining the .torrent file or parsing its contents is
       caught and wrapped in one of the following errors:
       GetTorrent.URLException, GetTorrent.FileException, 
       GetTorrent.MetainfoException, or GetTorrent.UnknownArgument.
       (All have the base class GetTorrent.GetTorrentException)
       """
    data = _get(arg)
    metainfo = None
    try:
        b = bdecode(data)
        metainfo = ConvertedMetainfo(b)
    except Exception, e:
        raise MetainfoException(
            (_('"%s" is not a valid torrent file (%s).') % (arg, unicode(e)))
            )

    return metainfo


def get_url(url):
    """Downloads the .torrent metainfo file specified by the passed
       URL and returns data, the raw contents of the metainfo file.
       Any exception raised while trying to obtain the metainfo file
       is caught and GetTorrent.URLException is raised instead.
       """

    data = None
    err_str = ((_('Could not download or open "%s"')% url) + '\n' +
               _("Try using a web browser to download the torrent file."))
    u = None

    # pending protocol changes, convert:
    #   torrent://http://path.to/file
    # and:
    #   bittorrent://http://path.to/file
    # to:
    #   http://path.to/file
    url = urlpat_torrent.sub('', url)
    url = urlpat_bittorrent.sub('', url)
    
    try:
        u = zurllib.urlopen(url)
        data = u.read()
        u.close()
    except Exception, e:
        if u is not None:
            u.close()
        raise URLException(err_str + "\n(%s)" % e)
    else:
        if u is not None:
            u.close()

    return data
    

def get_file(filename):
    """Reads the .torrent metainfo file specified by the passed
       filename and returns data, the raw contents of the metainfo file.
       Any exception raised while trying to obtain the metainfo file is
       caught and GetTorrent.FileException is raised instead.
       """
    data = None
    f = None
    try:
        f = file(filename, 'rb')
        data = f.read()
        f.close()
    except Exception, e:
        if f is not None:
            f.close()
        raise FileException((_("Could not read %s") % filename) + (': %s' % unicode(e.args[0])))

    return data



import os
import win32process
import win32con
import win32api

def compare(x, y):
    """ tries to fuzzy match process names """
    if x.lower() == y.lower():
        return True
    y = '.'.join(y.split('.')[:-1])
    if x.lower() == y.lower():
        return True
    return False

def kill_process(name):
    
    for pid in win32process.EnumProcesses():
        
        # do try not to kill yourself
        if pid == win32api.GetCurrentProcessId():
            continue
        
        try:
            p = win32api.OpenProcess(win32con.PROCESS_QUERY_INFORMATION
                                     | win32con.PROCESS_VM_READ
                                     | win32con.PROCESS_TERMINATE,
                                     False, pid)
        except:
            continue

        if not p:
            continue
        
        try:
            hl = win32process.EnumProcessModules(p)
        except:
            win32api.CloseHandle(p)
            continue

        h = hl[0]
        pname = win32process.GetModuleFileNameEx(p, h)
        root, pname = os.path.split(pname)
        #print name, pname
        if compare(name, pname):
            #print "KILL", pname
            win32api.TerminateProcess(p, 0)
            win32api.CloseHandle(p)
            return True

        win32api.CloseHandle(p)
    return False

if __name__ == '__main__':
    import sys
    n = sys.argv[1]
    r = kill_process(n)
    if r:
        print n, "killed"
    else:
        print n, "not killed"


# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen

from cStringIO import StringIO
from socket import error as socketerror

protocol_name = 'BitTorrent protocol'

# header, reserved, download id, my id, [length, message]


class NatCheck(object):

    def __init__(self, resultfunc, downloadid, peerid, ip, port, rawserver):
        self.resultfunc = resultfunc
        self.downloadid = downloadid
        self.peerid = peerid
        self.ip = ip
        self.port = port
        self.closed = False
        self.buffer = StringIO()
        self.next_len = 1
        self.next_func = self.read_header_len
        rawserver.start_connection((ip, port), self)
        

    def connection_made(self, s):
        self.connection = s
        self.connection.write(chr(len(protocol_name)) + protocol_name +
                              (chr(0) * 8) + self.downloadid)

    def connection_failed(self, addr, exception):
        self.answer(False)

    def answer(self, result):
        self.closed = True
        try:
            self.connection.close()
        except AttributeError:
            pass
        self.resultfunc(result, self.downloadid, self.peerid, self.ip, self.port)

    def read_header_len(self, s):
        if ord(s) != len(protocol_name):
            return None
        return len(protocol_name), self.read_header

    def read_header(self, s):
        if s != protocol_name:
            return None
        return 8, self.read_reserved

    def read_reserved(self, s):
        return 20, self.read_download_id

    def read_download_id(self, s):
        if s != self.downloadid:
            return None
        return 20, self.read_peer_id

    def read_peer_id(self, s):
        if s != self.peerid:
            return None
        self.answer(True)
        return None

    def data_came_in(self, connection, s):
        while True:
            if self.closed:
                return
            i = self.next_len - self.buffer.tell()
            if i > len(s):
                self.buffer.write(s)
                return
            self.buffer.write(s[:i])
            s = s[i:]
            m = self.buffer.getvalue()
            self.buffer.reset()
            self.buffer.truncate()
            x = self.next_func(m)
            if x is None:
                if not self.closed:
                    self.answer(False)
                return
            self.next_len, self.next_func = x

    def connection_lost(self, connection):
        if not self.closed:
            self.closed = True
            self.resultfunc(False, self.downloadid, self.peerid, self.ip, self.port)

    def connection_flushed(self, connection):
        pass



# Import and use just like threading, but enjoy the benefit of seeing the
# calling context prepended to any thread traceback.
#
# by Greg Hazel

import os        
import sys
import threading
import traceback
from threading import *

def _print(stream, str='', terminator='\n'):
    stream.write(str+terminator)

def _print_traceback(stream, base_stack, name, extra, exc_type, value, tb):
    stack2 = traceback.extract_tb(tb)
    # cut off the top
    stack2 = stack2[extra + 1:]
    base_stack.extend(stack2)
    l = traceback.format_list(base_stack)
    _print(stream, "Exception in %s:" % name)
    _print(stream, "Traceback (most recent call last):")
    for s in l:
        _print(stream, s, '')
    
    lines = traceback.format_exception_only(exc_type, value)
    for line in lines[:-1]:
        _print(stream, line, ' ')
    _print(stream, lines[-1], '')
    

base_Thread = Thread

class StackThread(Thread):

    def __init__(self, group=None, target=None, name=None, depth=1,
                 args=(), kwargs={}, verbose=None):

        if name is None:
            try:
                raise ZeroDivisionError
            except ZeroDivisionError:
                f = sys.exc_info()[2].tb_frame.f_back

            stack = traceback.extract_stack(f)
            fn, ln, fc, cd = stack[-depth]
            #sys.stdout.writelines([str(s)+'\n' for s in stack])
            root, fn = os.path.split(fn)
            name = '%s:%s in %s: %s' % (fn, ln, fc, cd)
            
        base_Thread.__init__(self, group=group, target=target, name=name,
                             args=args, kwargs=kwargs, verbose=verbose)
        
        stream = sys.stderr    

        start = self.start
        def save():
            # ha ha ha
            try:
                raise ZeroDivisionError
            except ZeroDivisionError:
                f = sys.exc_info()[2].tb_frame.f_back

            self.stack = traceback.extract_stack(f)
            start()
        
        run = self.run
        def catch():
            try:
                run()
            except:
                # interpreter shutdown
                if not sys:
                    return
                exc_type, value, tb = sys.exc_info()
                _print_traceback(stream, self.stack,
                                 "thread %s" % self.getName(), 1,
                                 exc_type, value, tb)

        self.run = catch
        self.start = save

Thread = StackThread
# this is annoying
_Event = threading._Event

if __name__ == '__main__':
    def foo():
        raise ValueError("boo!")

    t = Thread(target=foo)
    t.start()



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.
#
# Written by Matt Chisholm
import os
import imp
import traceback
import threading
from BitTorrent.platform import plugin_path
from BitTorrent import version

class BasePlugin(object):

    def __init__(self, main):
        self.main = main
        
    def _supports(version):
        return False

    supports = staticmethod(_supports)


class DownloadPlugin(BasePlugin):
    pass

PLUGIN_EXTENSION = '.py'

class PluginManager(object):
    kind = ''
    def __init__(self, config, ui_wrap_func):
        self.config = config
        self.ui_wrap_func = ui_wrap_func
        self.plugin_path = [os.path.join(x, self.kind) for x in plugin_path]
        self.plugins = []
        self._load_plugins()
        
    def _load_plugins(self):
        for p in self.plugin_path:
            files = os.listdir(p)
            for f in files:
                if f[0] != '_' and f.endswith(PLUGIN_EXTENSION):
                    filename = f[:-len(PLUGIN_EXTENSION)]
                    try:
                        plugin_module = imp.load_module(filename, *imp.find_module(filename, [p]))
                    except ImportError:
                        self.show_status('Could not load %s plugin' % filename)
                        traceback.print_exc()
                        continue
                    for c in dir(plugin_module):
                        if c[0] != '_':                        
                            plugin = getattr(plugin_module, c)
                            if self._check_plugin(plugin):
                                self.plugins.append(plugin)
        self.show_status("Loaded:", self.plugins)

    def _check_plugin(self, plugin):
        if not hasattr(plugin, 'supports'):
            return False
        if not plugin.supports(version):
            return False
        return True        

    def _find_plugin(self, *args):
        for p in self.plugins:
            if p.matches_type(*args):
                self.show_status('Found', p, 'for', args)
                return p
        return None

    def run_ui_task(self, funcname, *args):
        self.show_status('Would run', funcname, 'with', args, 'using', self.ui_wrap_func)

    def show_status(self, *msg):
        if False:
            print ' '.join(map(str, msg))



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# written by Matt Chisholm

import wx

from BitTorrent.defer import ThreadedDeferred
from BitTorrent import app_name, languages, language_names
from BitTorrent.platform import read_language_file, write_language_file

from BitTorrent.GUI_wx import SPACING, VSizer, gui_wrap, text_wrappable


error_color = wx.Colour(192,0,0)

class LanguageSettings(wx.Panel):

    def __init__(self, parent, *a, **k):
        wx.Panel.__init__(self, parent, *a, **k)
        self.sizer = VSizer()
        self.SetSizer(self.sizer)
        if 'errback' in k:
            self.errback = k.pop('errback')
        else:
            self.errback = self.set_language_failed

        # widgets
        self.box = wx.StaticBox(self, label="Translate %s into:" % app_name)

        self.language_names = ["System default",] + [language_names[l] for l in languages]
        languages.insert(0, '')
        self.languages = languages
        self.choice = wx.Choice(self, choices=self.language_names)
        self.Bind(wx.EVT_CHOICE, self.set_language, self.choice)

        restart = wx.StaticText(self, -1,
                                "You must restart %s for the\nlanguage "
                                "setting to take effect." % app_name)

        self.bottom_error = wx.StaticText(self, -1, '')
        self.bottom_error.SetForegroundColour(error_color)

        # sizers
        self.box_sizer = wx.StaticBoxSizer(self.box, wx.VERTICAL)

        # set menu selection and warning item if necessary
        self.valid = True
        lang = read_language_file()
        if lang is not None:
            try:
                i = self.languages.index(lang)
                self.choice.SetSelection(i)
            except ValueError, e:
                self.top_error = wx.StaticText(self, -1,
                                     "This version of %s does not \nsupport the language '%s'."%(app_name,lang),)
                self.top_error.SetForegroundColour(error_color)

                self.box_sizer.Add(self.top_error, flag=wx.TOP|wx.LEFT|wx.RIGHT, border=SPACING)
                # BUG add menu separator
                # BUG change color of extra menu item
                self.choice.Append(lang)
                self.choice.SetSelection(len(self.languages))
                self.valid = False
        else:
            self.choice.SetSelection(0)

        # other sizers
        self.box_sizer.Add(self.choice, flag=wx.GROW|wx.ALL, border=SPACING)
        self.box_sizer.Add(restart, flag=wx.BOTTOM|wx.LEFT|wx.RIGHT, border=SPACING)
        self.box_sizer.Add(self.bottom_error, flag=wx.BOTTOM|wx.LEFT|wx.RIGHT, border=SPACING)

        # clear out bottom error
        self.clear_error()

        self.sizer.AddFirst(self.box_sizer, flag=wx.GROW)
        self.sizer.Fit(self)


    def set_language(self, *a):
        index = self.choice.GetSelection()
        if index >= len(self.languages):
            return
        l = self.languages[index]
        if not self.valid:
            self.choice.Delete(len(self.languages))
            self.choice.SetSelection(index)
            self.valid = True
            self.box_sizer.Detach(0)
            self.top_error.Destroy()
            self.box_sizer.Layout()
            self.sizer.Layout()

        d = ThreadedDeferred(gui_wrap, write_language_file, l)
        d.addErrback(lambda e: self.set_language_failed(e, l))
        d.addCallback(lambda r: self.language_was_set())


    def language_was_set(self, *a):
        self.clear_error()
        wx.MessageBox("You must restart %s for the language "
                      "setting to take effect." % app_name,
                      "%s translation" % app_name,
                      style=wx.ICON_INFORMATION)

    def clear_error(self):
        index = self.box_sizer.GetItem(self.bottom_error)
        if index:
           self.box_sizer.Detach(self.bottom_error)
        self.bottom_error.SetLabel('')

        self.refit()

    def set_error(self, errstr):
        index = self.box_sizer.GetItem(self.bottom_error)
        if not index:
            self.box_sizer.Add(self.bottom_error, flag=wx.BOTTOM|wx.LEFT|wx.RIGHT, border=SPACING)
        self.bottom_error.SetLabel(errstr)
        if text_wrappable: self.bottom_error.Wrap(250)

        self.refit()


    def set_language_failed(self, e, l):
        # BUG: this really should be logged
        errstr = 'Could not find translation for language "%s"\n%s' % (l, str(e))
        self.set_error(errstr)


    def refit(self):
        self.box_sizer.Layout()
        self.sizer.Layout()
        #self.sizer.Fit(self)
        self.GetParent().Fit()



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

#!/usr/bin/env python

import os
import sys

try:
    import distutils.core
    import distutils.command.build_ext
except ImportError:
    raise SystemExit, """\
You don't have the python development modules installed.  

If you have Debian you can install it by running
    apt-get install python-dev

If you have RedHat and know how to install this from an RPM please
email us so we can put instructions here.
"""

try:
    import twisted
except ImportError:
    raise SystemExit, """\
You don't have Twisted installed.

Twisted can be downloaded from 
    http://twistedmatrix.com/products/download

Anything later that version 1.0.3 should work
"""

try:
    import sqlite
except ImportError:
    raise SystemExit, """\
You don't have PySQLite installed.

PySQLite can be downloaded from 
    http://sourceforge.net/project/showfiles.php?group_id=54058&release_id=139482
"""

setup_args = {
    'name': 'khashmir',
    'author': 'Andrew Loewenstern',
    'author_email': 'burris@users.sourceforge.net',
    'licence': 'MIT',
    'package_dir': {'khashmir': '.'},
    'packages': [
        'khashmir', 
    ],
}

if hasattr(distutils.dist.DistributionMetadata, 'get_keywords'):
    setup_args['keywords'] = "internet tcp p2p"

if hasattr(distutils.dist.DistributionMetadata, 'get_platforms'):
    setup_args['platforms'] = "win32 posix"

if __name__ == '__main__':
    apply(distutils.core.setup, (), setup_args)



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Greg Hazel

## up
# p.add(piece, p.remove(piece) + 1)
## down
# p.add(piece, p.remove(piece) - 1)

from BitTorrent.sparse_set import SparseSet

class PieceSetBuckets(object):
    """A PieceBuckets object is an array of arrays.  ith bucket contains
       pieces that have i known instances within the network.  Pieces
       within each bucket are randomly ordered."""
    def __init__(self):
        # [SparseSet(piece)]
        self.buckets = []
        # {piece: (bucket)}
        self.place_in_buckets = {}

    def get_position(self, piece):  # returns which bucket piece is in.
        return self.place_in_buckets[piece]

    def __contains__(self, piece):
        return piece in self.place_in_buckets

    def add(self, piece, bucketindex):
        assert not self.place_in_buckets.has_key(piece)
        while len(self.buckets) <= bucketindex:
            self.buckets.append(SparseSet())
        bucket = self.buckets[bucketindex]
        bucket.add(piece)
        self.place_in_buckets[piece] = bucketindex

    def remove(self, piece):
        bucketindex = self.place_in_buckets.pop(piece)
        bucket = self.buckets[bucketindex]
        bucket.subtract(piece)
        while len(self.buckets) > 0 and len(self.buckets[-1]) == 0:
            del self.buckets[-1]
        return bucketindex

    def prepend_bucket(self):
        # it' possible we had everything to begin with
        if len(self.buckets) == 0:
            return
        self.buckets.insert(0, SparseSet())
        # bleh.
        for piece in self.place_in_buckets:
            self.place_in_buckets[piece] += 1

    def popleft_bucket(self):
        # it' possible we had everything to begin with
        if len(self.buckets) == 0:
            return 
        self.buckets.pop(0)
        # bleh.
        for piece in self.place_in_buckets:
            self.place_in_buckets[piece] -= 1



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen, Uoti Urpala

from __future__ import division

class TorrentStats(object):

    def __init__(self, logger, choker, upfunc, downfunc, uptotal, downtotal,
                 remainingfunc, pcfunc, piece_states, finflag,
                 connection_manager, multidownload, file_priorities,
                 files, ever_got_incoming, rerequester):
        self.logger = logger
        self.multidownload = multidownload
        self.connection_manager = connection_manager
        self.file_priorities = file_priorities
        self.picker = multidownload.picker
        self.storage = multidownload.storage
        self.choker = choker
        self.connection_manager = connection_manager
        self.upfunc = upfunc
        self.downfunc = downfunc
        self.uptotal = uptotal
        self.downtotal = downtotal
        self.remainingfunc = remainingfunc
        self.pcfunc = pcfunc
        self.piece_states = piece_states
        self.finflag = finflag
        self.files = files
        self.ever_got_incoming = ever_got_incoming
        self.rerequester = rerequester

    def collect_spew(self):
        l = []
        for c in self.connection_manager.complete_connections:
            rec = {}
            rec['id'] = c.id
            rec["ip"] = c.ip
            rec["is_optimistic_unchoke"] = (c is self.choker.connections[0])

            if c.locally_initiated:
                rec["initiation"] = "L"
            else:
                rec["initiation"] = "R"
            if c._decrypt:
                rec["initiation"] += '+'

            u = c.upload
            rec["upload"] = (u.measure.get_total(), int(u.measure.get_rate()),
                             u.interested, u.choked)

            d = c.download
            rec["download"] = (d.measure.get_total(),int(d.measure.get_rate()),
                               d.interested, d.choked, d.is_snubbed())
            rec['completed'] = 1 - d.have.numfalse / len(d.have)
            rec['speed'] = d.connection.download.peermeasure.get_rate()
            if d.have.numfalse > 0:
                rec['total_eta'] = self.storage.total_length / max(1, d.connection.download.peermeasure.get_rate())
            l.append(rec)
        return l

    def get_swarm_speed(self):
        speeds = []
        for c in self.connection_manager.complete_connections:
            d = c.download
            speeds.append(d.connection.download.peermeasure.get_rate())
        speed = sum(speeds)
        return speed

    def get_statistics(self, spewflag=False, fileflag=False):
        status = {}

        numSeeds = 0
        numPeers = 0
        for d in self.multidownload.downloads:
            numPeers += 1
            if d.have.numfalse == 0:
                numSeeds += 1
        status['numSeeds'] = numSeeds
        status['numPeers'] = numPeers

        if self.rerequester:
            status['trackerSeeds'] = self.rerequester.tracker_num_seeds
            status['trackerPeers'] = self.rerequester.tracker_num_peers
            if status['trackerSeeds'] is not None:
                if status['trackerPeers'] is not None:
                    status['trackerPeers'] += status['trackerSeeds']
                else:
                    status['trackerPeers'] = status['trackerSeeds']

            status['announceTime'] = self.rerequester.get_next_announce_time_est()
        else:
            status['trackerSeeds'] = None
            status['trackerPeers'] = None
            status['announceTime'] = None
        status['upRate'] = self.upfunc()
        status['upTotal'] = self.uptotal()
        status['ever_got_incoming'] = self.ever_got_incoming()

        status['distributed_copies'] = self.multidownload.get_adjusted_distributed_copies()

        status['discarded'] = self.multidownload.discarded_bytes


        status['swarm_speed'] = self.get_swarm_speed()

        status['pieceStates'] = self.piece_states()

        if spewflag:
            status['spew'] = self.collect_spew()
            status['bad_peers'] = self.multidownload.bad_peers
        if fileflag:
            undl = self.storage.storage.undownloaded
            status['files_left'] = [undl[fname] for fname in self.files]
            status['file_priorities'] = dict(self.file_priorities())
        if self.finflag.isSet():
            status['downRate'] = 0
            status['downTotal'] = self.downtotal()
            status['fractionDone'] = 1
            return status
        timeEst = self.remainingfunc()
        status['timeEst'] = timeEst

        fractionDone = self.pcfunc()
        status.update({
            "fractionDone" : fractionDone,
            "downRate" : self.downfunc(),
            "downTotal" : self.downtotal()
            })
        return status



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Uoti Urpala

# required for Python 2.2
from __future__ import generators

import os
import sys
import logging
import urlparse
from BitTorrent.hash import sha
import socket

#debug=True
global_logger = logging.getLogger("BitTorrent.ConvertedMetainfo")

from BitTorrent.translation import _
from BitTorrent.obsoletepythonsupport import *

from BitTorrent.bencode import bencode
from BitTorrent import btformats
from BitTorrent import BTFailure, InfoHashType
from BitTorrent.platform import get_filesystem_encoding, encode_for_filesystem
from BitTorrent.defer import ThreadedDeferred

WINDOWS_UNSUPPORTED_CHARS = u'"*/:<>?\|'
windows_translate = {}
for x in WINDOWS_UNSUPPORTED_CHARS:
    windows_translate[ord(x)] = u'-'

noncharacter_translate = {}
for i in xrange(0xD800, 0xE000):
    noncharacter_translate[i] = ord('-')
for i in xrange(0xFDD0, 0xFDF0):
    noncharacter_translate[i] = ord('-')
for i in (0xFFFE, 0xFFFF):
    noncharacter_translate[i] = ord('-')

del x, i

def generate_names(name, is_dir):
    if is_dir:
        prefix = name + '.'
        suffix = ''
    else:
        pos = name.rfind('.')
        if pos == -1:
            pos = len(name)
        prefix = name[:pos] + '.'
        suffix = name[pos:]
    i = 0
    while True:
        yield prefix + str(i) + suffix
        i += 1

class ConvertedMetainfo(object):

    def __init__(self, metainfo):
        """metainfo is a dict.  When read from a metainfo (i.e.,
           .torrent file), the file must first be bdecoded before
           being passed to ConvertedMetainfo."""
        self.bad_torrent_wrongfield = False
        self.bad_torrent_unsolvable = False
        self.bad_torrent_noncharacter = False
        self.bad_conversion = False
        self.bad_windows = False
        self.bad_path = False
        self.reported_errors = False
        self.is_batch = False
        self.orig_files = None
        self.files_fs = None
        self.total_bytes = 0
        self.sizes = []
        self.comment = None
        self.title = None          # descriptive title text for whole torrent
        self.creation_date = None
        self.metainfo = metainfo
        self.encoding = None
        self.caches = None

        btformats.check_message(metainfo, check_paths=False)
        info = metainfo['info']
        self.is_private = info.has_key("private") and info['private']
        if 'encoding' in metainfo:
            self.encoding = metainfo['encoding']
        elif 'codepage' in metainfo:
            self.encoding = 'cp%s' % metainfo['codepage']
        if self.encoding is not None:
            try:
                for s in u'this is a test', u'these should also work in any encoding: 0123456789\0':
                    assert s.encode(self.encoding).decode(self.encoding) == s
            except:
                self.encoding = 'iso-8859-1'
                self.bad_torrent_unsolvable = True
        if info.has_key('length'):
            self.total_bytes = info['length']
            self.sizes.append(self.total_bytes)
        else:
            self.is_batch = True
            r = []
            self.orig_files = []
            self.sizes = []
            i = 0
            for f in info['files']:
                l = f['length']
                self.total_bytes += l
                self.sizes.append(l)
                path = self._get_attr(f, 'path')
                if len(path[-1]) == 0:
                    if l > 0:
                        raise BTFailure(_("Bad file path component: ")+x)
                    # BitComet makes .torrent files with directories
                    # listed along with the files, which we don't support
                    # yet, in part because some idiot interpreted this as
                    # a bug in BitComet rather than a feature.
                    path.pop(-1)

                for x in path:
                    if not btformats.allowed_path_re.match(x):
                        raise BTFailure(_("Bad file path component: ")+x)

                self.orig_files.append('/'.join(path))
                k = []
                for u in path:
                    tf2 = self._to_fs_2(u)
                    k.append((tf2, u))
                r.append((k,i))
                i += 1
            # If two or more file/subdirectory names in the same directory
            # would map to the same name after encoding conversions + Windows
            # workarounds, change them. Files are changed as
            # 'a.b.c'->'a.b.0.c', 'a.b.1.c' etc, directories or files without
            # '.' as 'a'->'a.0', 'a.1' etc. If one of the multiple original
            # names was a "clean" conversion, that one is always unchanged
            # and the rest are adjusted.
            r.sort()
            self.files_fs = [None] * len(r)
            prev = [None]
            res = []
            stack = [{}]
            for x in r:
                j = 0
                x, i = x
                while x[j] == prev[j]:
                    j += 1
                del res[j:]
                del stack[j+1:]
                name = x[j][0][1]
                if name in stack[-1]:
                    for name in generate_names(x[j][1], j != len(x) - 1):
                        name = self._to_fs(name)
                        if name not in stack[-1]:
                            break
                stack[-1][name] = None
                res.append(name)
                for j in xrange(j + 1, len(x)):
                    name = x[j][0][1]
                    stack.append({name: None})
                    res.append(name)
                self.files_fs[i] = os.path.join(*res)
                prev = x

        self.name = self._get_attr(info, 'name')
        self.name_fs = self._to_fs(self.name)
        self.piece_length = info['piece length']

        self.announce = metainfo.get('announce')
        self.announce_list = metainfo.get('announce-list')
        if 'announce-list' not in metainfo and 'announce' not in metainfo:
            self.is_trackerless = True
        else:
            self.is_trackerless = False

        self.nodes = metainfo.get('nodes')

        self.title = metainfo.get('title')
        self.comment = metainfo.get('comment')
        self.creation_date = metainfo.get('creation date')
        self.locale = metainfo.get('locale')

        self.url_list = metainfo.get('url-list', [])
        if not isinstance(self.url_list, list):
            self.url_list = [self.url_list, ]

        self.caches = metainfo.get('caches')

        self.hashes = [info['pieces'][x:x+20] for x in xrange(0,
            len(info['pieces']), 20)]
        self.infohash = InfoHashType(sha(bencode(info)).digest())


    def show_encoding_errors(self, errorfunc):
        self.reported_errors = True
        if self.bad_torrent_unsolvable:
            errorfunc(logging.ERROR,
                      _("This .torrent file has been created with a broken "
                        "tool and has incorrectly encoded filenames. Some or "
                        "all of the filenames may appear different from what "
                        "the creator of the .torrent file intended."))
        elif self.bad_torrent_noncharacter:
            errorfunc(logging.ERROR,
                      _("This .torrent file has been created with a broken "
                        "tool and has bad character values that do not "
                        "correspond to any real character. Some or all of the "
                        "filenames may appear different from what the creator "
                        "of the .torrent file intended."))
        elif self.bad_torrent_wrongfield:
            errorfunc(logging.ERROR,
                      _("This .torrent file has been created with a broken "
                        "tool and has incorrectly encoded filenames. The "
                        "names used may still be correct."))
        elif self.bad_conversion:
            errorfunc(logging.WARNING,
                      _('The character set used on the local filesystem ("%s") '
                        'cannot represent all characters used in the '
                        'filename(s) of this torrent. Filenames have been '
                        'changed from the original.') % get_filesystem_encoding())
        elif self.bad_windows:
            errorfunc(logging.WARNING,
                      _("The Windows filesystem cannot handle some "
                        "characters used in the filename(s) of this torrent. "
                        "Filenames have been changed from the original."))
        elif self.bad_path:
            errorfunc(logging.WARNING,
                      _("This .torrent file has been created with a broken "
                        "tool and has at least 1 file with an invalid file "
                        "or directory name. However since all such files "
                        "were marked as having length 0 those files are "
                        "just ignored."))

    # At least BitComet seems to make bad .torrent files that have
    # fields in an unspecified non-utf8 encoding.  Some of those have separate
    # 'field.utf-8' attributes.  Less broken .torrent files have an integer
    # 'codepage' key or a string 'encoding' key at the root level.
    def _get_attr(self, d, attrib):
        def _decode(o, encoding):
            if encoding is None:
                encoding = 'utf8'
            if isinstance(o, str):
                try:
                    s = o.decode(encoding)
                except:
                    self.bad_torrent_wrongfield = True
                    s = o.decode(encoding, 'replace')
                t = s.translate(noncharacter_translate)
                if t != s:
                    self.bad_torrent_noncharacter = True
                return t
            if isinstance(o, dict):
                return dict([ (k, _decode(v, k.endswith('.utf-8') and None or encoding)) for k, v in o.iteritems() ])
            if isinstance(o, list):
                return [ _decode(i, encoding) for i in o ]
            return o
        # we prefer utf8 if we can find it. at least it declares its encoding
        v = _decode(d.get(attrib + '.utf-8'), 'utf8')
        if v is None:
            v = _decode(d[attrib], self.encoding)
        return v

    def _fix_windows(self, name, t=windows_translate):
        bad = False
        r = name.translate(t)
        # for some reason name cannot end with '.' or space
        if r[-1] in '. ':
            r = r + '-'
        if r != name:
            self.bad_windows = True
            bad = True
        return (r, bad)

    def _to_fs(self, name):
        return self._to_fs_2(name)[1]

    def _to_fs_2(self, name):
        if sys.platform.startswith('win'):
            name, bad = self._fix_windows(name)

        r, bad = encode_for_filesystem(name)
        self.bad_conversion = bad

        return (bad, r)


    def to_data(self):
        return bencode(self.metainfo)


    def check_for_resume(self, path):
        """
        Determine whether this torrent was previously downloaded to
        path.  Returns:

        -1: STOP! gross mismatch of files
         0: MAYBE a resume, maybe not
         1: almost definitely a RESUME - file contents, sizes, and count match exactly
        """
        STOP   = -1
        MAYBE  =  0
        RESUME =  1

        if self.is_batch != os.path.isdir(path):
            return STOP

        disk_files = {}
        if self.is_batch:
            metainfo_files = dict(zip(self.files_fs, self.sizes))
            metainfo_dirs = set()
            for f in self.files_fs:
                metainfo_dirs.add(os.path.split(f)[0])

            # BUG: do this in a thread, so it doesn't block the UI
            for (dirname, dirs, files) in os.walk(path):
                here = dirname[len(path)+1:]
                for f in files:
                    p = os.path.join(here, f)
                    if p in metainfo_files:
                        disk_files[p] = os.stat(os.path.join(dirname, f))[6]
                        if disk_files[p] > metainfo_files[p]:
                            # file on disk that's bigger than the
                            # corresponding one in the torrent
                            return STOP
                    else:
                        # file on disk that's not in the torrent
                        return STOP
                for i, d in enumerate(dirs):
                    if d not in metainfo_dirs:
                        # directory on disk that's not in the torrent
                        return STOP

        else:
            if os.access(path, os.F_OK):
                disk_files[self.name_fs] = os.stat(path)[6]
            metainfo_files = {self.name_fs : self.sizes[0]}

        if len(disk_files) == 0:
            # no files on disk, definitely not a resume
            return STOP

        if set(disk_files.keys()) != set(metainfo_files.keys()):
            # check files
            if len(metainfo_files) > len(disk_files):
                #file in the torrent that's not on disk
                return MAYBE
        else:
            # check sizes
            ret = RESUME
            for f, s in disk_files.iteritems():

                if disk_files[f] < metainfo_files[f]:
                    # file on disk that's smaller than the
                    # corresponding one in the torrent
                    ret = MAYBE
                else:
                    # file sizes match exactly
                    continue
            return ret

    def get_tracker_ips(self, wrap_task):
        """Returns the list of tracker IP addresses or the empty list if the
           torrent is trackerless.  This extracts the tracker ip addresses
           from the urls in the announce or announce list."""
        df = ThreadedDeferred(wrap_task, self._get_tracker_ips, daemon=True)
        return df

    def _get_tracker_ips(self):
        if hasattr(self, "_tracker_ips"):     # cache result.
            return self._tracker_ips

        if self.announce is not None:
            urls = [self.announce]
        elif self.announce_list is not None:  # list of lists.
            urls = []
            for ulst in self.announce_list:
                urls.extend(ulst)
        else:  # trackerless
            assert self.is_trackerless
            return []

        tracker_ports = [urlparse.urlparse(url)[1] for url in urls]
        trackers = [tp.split(':')[0] for tp in tracker_ports]
        self._tracker_ips = []
        for t in trackers:
            try:
                ip_list = socket.gethostbyname_ex(t)[2]
                self._tracker_ips.extend(ip_list)
            except socket.gaierror:
                global_logger.error( _("Cannot find tracker with name %s") % t )
        return self._tracker_ips






# by Greg Hazel

from __future__ import generators

from struct import pack, unpack

import urllib
import logging

from BitTorrent.obsoletepythonsupport import *
from BitTorrent.DictWithLists import OrderedDict
from BitTorrent.Connector import Connection
from bisect import bisect_right

# package management sucks!
# it encourages code duplication because it can't keep up
# with packages that have been out for over a year.
# as a result, this is copy'n'pasted twisted 2.0 code
#from twisted.web.http import parseContentRange
def parseContentRange(header):
    """Parse a content-range header into (start, end, realLength).

    realLength might be None if real length is not known ('*').
    """
    kind, other = header.strip().split()
    if kind.lower() != "bytes":
        raise ValueError, "a range of type %r is not supported"
    startend, realLength = other.split("/")
    start, end = map(int, startend.split("-"))
    if realLength == "*":
        realLength = None
    else:
        realLength = int(realLength)
    return (start, end, realLength)

  
noisy = True
if noisy:
    connection_logger = logging.getLogger("BitTorrent.HTTPConnector")
    def log(s):
        print s
    #log = connection_logger.debug


def protocol_violation(s, c=None):
    a = ''
    if noisy:
        if c is not None:
            a = (c.ip, c.port)
        log( "FAUX PAS: %s %s" % ( s, a ))

class BatchRequests(object):

    def __init__(self):
        self.requests = {}

    # you should add from the perspective of a BatchRequest
    def _add_request(self, filename, begin, length, br):
        r = (filename, begin, length)
        assert r not in self.requests
        self.requests[r] = br

    def got_request(self, filename, begin, data):
        length = len(data)
        r = (filename, begin, length)
        br = self.requests.pop(r)
        br.got_request(filename, begin, length, data)
        return br
        

class BatchRequest(object):
    
    def __init__(self, parent, start):
        self.parent = parent
        self.numactive = 0
        self.start = start
        self.requests = OrderedDict()

    def add_request(self, filename, begin, length):
        r = (filename, begin, length)
        assert r not in self.requests
        self.parent._add_request(filename, begin, length, self)
        self.requests[r] = None
        self.numactive += 1

    def got_request(self, filename, begin, length, data):
        self.requests[(filename, begin, length)] = data
        self.numactive -= 1

    def get_result(self):
        if self.numactive > 0:
            return None
        chunks = []
        for k in self.requests.itervalues():
            chunks.append(k)
        return ''.join(chunks)
        

# kind of like storage wrapper for webserver interaction
class URLage(object):

    def __init__(self, files):
        # a list of bytes ranges and filenames for window-based IO
        self.ranges = []
        self._build_url_structs(files)
        
    def _build_url_structs(self, files):
        total = 0
        for filename, length in files:
            if length > 0:
                self.ranges.append((total, total + length, filename))
            total += length
        self.total_length = total

    def _intervals(self, pos, amount):
        r = []
        stop = pos + amount
        p = max(bisect_right(self.ranges, (pos, )) - 1, 0)
        for begin, end, filename in self.ranges[p:]:
            if begin >= stop:
                break
            r.append((filename, max(pos, begin) - begin, min(end, stop) - begin))
        return r

    def _request(self, host, filename, pos, amount):
        b = pos
        e = b + amount - 1
        s = '\r\n'.join([
            "GET /%s HTTP/1.1" % (urllib.quote(filename)),
            "Host: %s" % host,
            "Connection: Keep-Alive",
            "Range: bytes=%s-%s" % (b, e),
            "", ""])
        return s    

    def build_requests(self, brs, host, pos, amount):
        r = []
        br = BatchRequest(brs, pos)
        for filename, pos, end in self._intervals(pos, amount):
            s = self._request(host, filename, pos, end - pos)
            br.add_request(filename, pos, end - pos)
            r.append((filename, s))
        return r
        

class HTTPConnection(Connection):
    """Implements the HTTP syntax with a BitTorrent Connection interface. 
       Connection-level semantics are as normal, but the download is always
       unchoked after it's connected."""

    MAX_LINE_LENGTH = 16384

    def __init__(self, parent, piece_size, urlage, *a, **kw):
        self.piece_size = piece_size
        self._header_lines = []
        self.manual_close = False
        self.urlage = urlage
        self.batch_requests = BatchRequests()
        # pipeline tracker
        self.request_paths = []
        Connection.__init__(self, parent, *a, **kw)

    def close(self):
        self.manual_close = True
        Connection.close(self)

    def send_handshake(self):
        self.send_request(0, 0, 1)

    def send_request(self, index, begin, length):
        if noisy:
            log( "SEND %s %d %d %d" % ('GET', index, begin, length) )
        b = (index * self.piece_size) + begin
        r = self.urlage.build_requests(self.batch_requests, self.id, b, length)
        for filename, s in r:
            self.request_paths.append(filename)
            if self._partial_message is not None:
                self._outqueue.append(s)
            else:
                self.connection.write(s)

    def send_interested(self):
        pass

    def send_not_interested(self):
        pass

    def send_choke(self):
        self.choke_sent = self.upload.choked

    def send_unchoke(self):
        self.choke_sent = self.upload.choked

    def send_cancel(self, index, begin, length):
        pass

    def send_have(self, index):
        pass

    def send_bitfield(self, bitfield):
        pass
    
    def send_keepalive(self):
        # is there something I can do here?
        pass

    # yields the number of bytes it wants next, gets those in self._message
    def _read_messages(self):
        completing = False
        
        while True:
            self._header_lines = []

            yield None
            if not self._message.startswith('http/1.1 206'):
                protocol_violation('Bad status message: %s' % self._message,
                                   self.connection)
                return

            if not self.complete:
                completing = True

            headers = {}
            while True:
                yield None
                if len(self._message) == 0:
                    break
                if ':' not in self._message:
                    protocol_violation('Bad header: %s' % self._message,
                                       self.connection)
                    return
                header, value = self._message.split(':', 1)
                headers[header] = value.strip()
            # reset the header buffer so we can loop
            self._header_lines = []

            filename = self.request_paths.pop(0)
            
            start, end, realLength = parseContentRange(headers['content-range'])
            length = (end - start) + 1
            cl = int(headers.get('content-length', length))
            assert (cl == length,
                    'Got c-l:%d bytes instead of l:%d' % (cl, length))
            yield length
            assert (len(self._message) == length,
                    'Got m:%d bytes instead of l:%d' % (len(self._message), length))
            
            if completing:
                self.complete = True
                completing = False
                print 'Completed', self
                self.parent.connection_completed(self)
                # prefer full pieces to reduce http overhead
                self.download.prefer_full = True
                self.download._got_have_all()
                self.download.got_unchoke()
            elif self.complete:
                self.got_anything = True
                br = self.batch_requests.got_request(filename, start, self._message)
                data = br.get_result()
                if data:
                    index = br.start // self.piece_size
                    if index >= self.parent.numpieces:
                        return
                    begin = br.start - (index * self.piece_size)
                    self.download.got_piece(index, begin, data)

    def data_came_in(self, conn, s):
        self.received_data = True
        if not self.download:
            # this is really annoying.
            self.sloppy_pre_connection_counter += len(s)
        else:
            l = self.sloppy_pre_connection_counter + len(s)
            self.sloppy_pre_connection_counter = 0

        self._buffer.append(s)
        self._buffer_len += len(s)
        # not my favorite loop.
        # the goal is: read self._next_len bytes, or if it's None return all
        # data up to a \r\n
        while True:
            if self.closed:
                return
            if self._next_len == None:
                if self._header_lines:
                    d = ''.join(self._buffer)
                    m = self._header_lines.pop(0).lower()
                else:
                    if '\r\n' not in s:
                        return
                    d = ''.join(self._buffer)
                    headers = d.split('\r\n')
                    # the last one is always trash
                    headers.pop(-1)
                    self._header_lines.extend(headers)
                    m = self._header_lines.pop(0).lower()
                if len(m) > self.MAX_LINE_LENGTH:
                    protocol_violation('Line length exceeded.',
                                       self.connection)
                    self.close()
                    return
                self._next_len = len(m) + len('\r\n')
            else:
                if self._next_len > self._buffer_len:
                    return
                d = ''.join(self._buffer)
                m = d[:self._next_len]
            s = d[self._next_len:]
            self._buffer = [s]
            self._buffer_len = len(s)
            self._message = m
            try:
                self._next_len = self._reader.next()
            except StopIteration:
                self.close()
                return

    def _optional_restart(self):            
        if self.complete and not self.manual_close:
            # http keep-alive has a per-connection limit on the number of requests
            # also, it times out. both result it a dropped connection, so re-make it.
            # idealistically, the connection would hang around even if dropped, and
            # reconnect if we needed to make a new request (that way we don't thrash
            # the piece picker everytime we reconnect)
            dns = (self.connection.ip, self.connection.port)
            self.parent.start_http_connection(dns, id=self.id)



_ = _ # put _ into the module namespace so the console doesn't override it
from twisted.web.xmlrpc import Proxy, XMLRPC, Fault, Binary
from BitTorrent.prefs import Preferences
from BitTorrent.NewTorrentQueue import TorrentQueue, TorrentAlreadyInQueue, TorrentAlreadyRunning, TorrentNotRunning, UnknownInfohash, TorrentShutdownFailed
from BitTorrent.ConvertedMetainfo import ConvertedMetainfo
from BitTorrent.bencode import bdecode

TORRENT_ALREADY_STORED = (201, _("Torrent already on disk."))
TORRENT_ALREADY_CREATED = (203, _("Torrent already created and under control of the seed."))
UNKNOWN_TORRENT = (202,  _("Unknown Torrent"))
TORRENT_NOT_RUNNING = (204, _("Torrent is not running."))
TORRENT_SHUTDOWN_FAILED = (205, _("Torrent shut down failed."))

class XMLRPCTQ(XMLRPC):
    def __init__(self, config, torrent_queue, logger):
        XMLRPC.__init__(self)
        self.config = Preferences(config)
        self.log = logger
        self.port = self.config['xmlrpc_port']
        self.tq = torrent_queue
        
    def xmlrpc_create_torrent(self, torrent, save_incomplete_as, 
                              save_as, shouldstart=False):
        try:
            infohash = self._create_torrent(torrent.data, save_incomplete_as,
                                            save_as)
        except (TorrentAlreadyRunning, TorrentAlreadyInQueue):
            return Fault(*TORRENT_ALREADY_CREATED)
        
        if shouldstart:
            self._start_torrent(infohash)
            
        return Binary(infohash)
    
    def _create_torrent(self, torrent, dlpath):
        d = bdecode(torrent)
        metainfo = ConvertedMetainfo(d)
        self.tq.create_torrent(metainfo, dlpath)
        return metainfo.infohash

    def xmlrpc_start_torrent(self, infohash):
        try:
            df = self._start_torrent(infohash.data)
        except UnknownInfohash:
            return Fault(*UNKNOWN_TORRENT)
        df.addCallback(lambda a: 0)
        return df

    def _start_torrent(self, infohash):
        return self.tq.start_torrent(infohash)
    
    def xmlrpc_stop_torrent(self, infohash):
        infohash = infohash.data
        try:
            self.tq.stop_torrent(infohash)
        except UnknownInfohash:
            return Fault(*UNKNOWN_TORRENT)
        except TorrentNotRunning:
            return Fault(*TORRENT_NOT_RUNNING)
        except TorrentShutdownFailed:
            return Fault(*TORRENT_SHUTDOWN_FAILED)
        return 0
        
    def xmlrpc_get_torrents(self):
         ### returns a list of all torrents in queue order, followed by a list running hashes
        return ([Binary(x) for x in self.tq.get_torrents()], [Binary(x) for x in self.tq.get_running()])
    
    def xmlrpc_get_status(self, infohash, spew=False, fileinfo=False):
        infohash = infohash.data
        try:
            d = self.tq.torrent_status(infohash, spew, fileinfo)
        except UnknownInfohash:
            return Fault(*UNKNOWN_TORRENT)
        except TorrentNotRunning:
            return Fault(*TORRENT_NOT_RUNNING)                
        n = {}
        for key, value in d.iteritems():
            if value != None:
                n[key] = value
        return n

    def xmlrpc_get_log(self, infohash, clear=True):
        try:
            l = self.tq.torrent_log(infohash, clear)
        except UnknownInfohash:
            return Fault(*UNKNOWN_TORRENT)
        except TorrentNotRunning:
            return Fault(*TORRENT_NOT_RUNNING)                
        return l

    def xmlrpc_get_global_log(self, clear=True):
        return self.tq.get_log(clear)
    
    def xmlrpc_set_torrent_option(self, infohash, option, value):
        infohash = infohash.data
        try:
            self.tq.set_config(self, option, value, infohash)
        except UnknownInfohash:
            return Fault(*UNKNOWN_TORRENT)
        return 0
    



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Bram Cohen and Greg Hazel

import array
import random
import itertools

def resolve_typecode(n):
    if n < 32768:
        return 'h'
    return 'l'

class PieceBuckets(object):
    """A PieceBuckets object is an array of arrays.  ith bucket contains
       pieces that have i known instances within the network.  Pieces
       within each bucket are randomly ordered."""
    def __init__(self, typecode):
        self.typecode = typecode
        # [[piece]]
        self.buckets = []
        # {piece: (bucket, bucketpos)}
        self.place_in_buckets = {}

    def get_position(self, piece):  # returns which bucket piece is in.
        return self.place_in_buckets[piece][0]

    def __contains__(self, piece):
        return piece in self.place_in_buckets

    def add(self, piece, bucketindex):
        assert not self.place_in_buckets.has_key(piece)
        while len(self.buckets) <= bucketindex:
            self.buckets.append(array.array(self.typecode))
        mylist = self.buckets[bucketindex]
        # randomly swap piece with piece already in bucket...
        newspot = random.randrange(len(mylist) + 1)
        if newspot == len(mylist):
            mylist.append(piece)
        else:
            tomove = mylist[newspot]
            self.place_in_buckets[tomove] = (bucketindex, len(mylist))
            mylist.append(tomove)
            mylist[newspot] = piece
        self.place_in_buckets[piece] = (bucketindex, newspot)

    def remove(self, piece):
        bucketindex, bucketpos = self.place_in_buckets.pop(piece)
        bucket = self.buckets[bucketindex]
        tomove = bucket[-1]
        if tomove != piece:
            bucket[bucketpos] = tomove
            self.place_in_buckets[tomove] = (bucketindex, bucketpos)
        del bucket[-1]
        while len(self.buckets) > 0 and len(self.buckets[-1]) == 0:
            del self.buckets[-1]
        return bucketindex

    # to be removed
    def bump(self, piece):
        bucketindex, bucketpos = self.place_in_buckets[piece]
        bucket = self.buckets[bucketindex]
        tomove = bucket[-1]
        if tomove != piece:
            bucket[bucketpos] = tomove
            self.place_in_buckets[tomove] = (bucketindex, bucketpos)
            bucket[-1] = piece
            self.place_in_buckets[piece] = (bucketindex, len(bucket)-1)

    def prepend_bucket(self):
        # it' possible we had everything to begin with
        if len(self.buckets) == 0:
            return
        self.buckets.insert(0, array.array(self.typecode))
        # bleh.
        for piece in self.place_in_buckets:
            index, pos = self.place_in_buckets[piece]
            self.place_in_buckets[piece] = (index + 1, pos)

    def popleft_bucket(self):
        # it' possible we had everything to begin with
        if len(self.buckets) == 0:
            return
        self.buckets.pop(0)
        # bleh.
        for piece in self.place_in_buckets:
            index, pos = self.place_in_buckets[piece]
            self.place_in_buckets[piece] = (index - 1, pos)

class PiecePicker(object):

    def __init__(self, config, numpieces, not_have):
        self.config = config
        self.numpieces = numpieces
        self.typecode = resolve_typecode(numpieces)
        self.piece_bucketss = [PieceBuckets(self.typecode)]
        self.scrambled = array.array(self.typecode)
        self.numgot = self.numpieces
        for i in not_have:
            self.scrambled.append(i)
            self.piece_bucketss[0].add(i, 0)
            self.numgot -= 1
        random.shuffle(self.scrambled)

    def get_distributed_copies(self):
        base = 0
        for i, bucket in enumerate(self.piece_bucketss[0].buckets):
            l = len(bucket)
            if l == 0:
                # the whole bucket is full. keep going
                continue
            base = i + 1
            # remove the fractional size of this bucket, and stop
            base -= (float(l) / float(self.numpieces))
            break
        return base

    def set_priority(self, pieces, priority):
        while len(self.piece_bucketss) <= priority:
            self.piece_bucketss.append(PieceBuckets(self.typecode))
        for piece in pieces:
            for p in self.piece_bucketss:
                if piece in p:
                    self.piece_bucketss[priority].add(piece, p.remove(piece))
                break
            else:
                assert False

    def got_have_all(self):
        for p in self.piece_bucketss:
            p.prepend_bucket()

    def got_have(self, piece):
        for p in self.piece_bucketss:
            if piece in p:
                p.add(piece, p.remove(piece) + 1)
                return

    def lost_have_all(self):
        for p in self.piece_bucketss:
            p.popleft_bucket()

    def lost_have(self, piece):
        for p in self.piece_bucketss:
            if piece in p:
                p.add(piece, p.remove(piece) - 1)
                return

    def complete(self, piece):
        self.numgot += 1
        if self.numgot < self.config['rarest_first_cutoff']:
            self.scrambled.remove(piece)
        else:
            self.scrambled = None
        for p in self.piece_bucketss:
            if piece in p:
                p.remove(piece)
                break
        else:
            assert False
       

    def from_behind(self, haves, bans):
        for piece_buckets in self.piece_bucketss:
            for i in xrange(len(piece_buckets.buckets) - 1, 0, -1):
                for j in piece_buckets.buckets[i]:
                    if haves[j] and j not in bans:
                        return j
        return None

    def next(self, haves, tiebreaks, bans, suggests):
        """returns next piece to download.
           @param haves: set of pieces the remote peer has.
           @param tiebreaks: pieces with active (started) requests
           @param bans: pieces not to pick.
           @param suggests: set of suggestions made by the remote peer.
        """
        # first few pieces are provided in random rather than rarest-first 
        if self.numgot < self.config['rarest_first_cutoff']:
            for i in itertools.chain(tiebreaks, self.scrambled):
                if haves[i] and i not in bans:
                    return i
            return None

        # from highest priority to lowest priority piece buckets...
        for k in xrange(len(self.piece_bucketss) - 1, -1, -1):

            piece_buckets = self.piece_bucketss[k]

            # Of the same priority, a suggestion is taken first.
            for s in suggests:
                if s not in bans and have(s) and s in piece_buckets:
                     return s

            bestnum = None
            best = None
            rarity_of_started = [(piece_buckets.get_position(i), i)
                    for i in tiebreaks if i not in bans and haves[i] and
                                          i in piece_buckets]
            if rarity_of_started:
                bestnum = min(rarity_of_started)[0]  # smallest bucket index
                best = random.choice([j for (i, j) in rarity_of_started
                    if i == bestnum]) # random pick of those in smallest bkt
                                                      
            for i in xrange(1, len(piece_buckets.buckets)):
                if bestnum == i:      # if best of started is also rarest...
                    return best
                for j in piece_buckets.buckets[i]:
                    if haves[j] and j not in bans:
                        return j      # return first found.
        return None
    
    # to be removed
    def bump(self, piece):
        for p in self.piece_bucketss:
            if piece in p:
                p.bump(piece)
                break
        else:
            assert False



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# written by Matt Chisholm

from __future__ import division

import os
import os.path
import math
import atexit
import itertools
import webbrowser
from copy import copy
import logging
import logging.handlers

from BitTorrent.translation import _

import BitTorrent.stackthreading as threading
from BitTorrent import GetTorrent
from BitTorrent import LaunchPath
from BitTorrent.MultiTorrent import UnknownInfohash, TorrentAlreadyInQueue, TorrentAlreadyRunning, TorrentNotRunning
from BitTorrent.obsoletepythonsupport import set
from BitTorrent.yielddefer import launch_coroutine
from BitTorrent.defer import ThreadedDeferred
from BitTorrent.platform import desktop, bttime
from BitTorrent.Torrent import *
from BitTorrent.ThreadProxy import ThreadProxy


state_dict = {("created", "stop", False): _("Paused"),
              ("created", "stop", True): _("Paused"),
              ("created", "start", False): _("Starting"),
              ("created", "start", True): _("Starting"),
              ("created", "auto", False): _("Starting"),
              ("created", "auto", True): _("Starting"),
              ("initializing", "stop", False): _("Paused"),
              ("initializing", "stop", True): _("Paused"),
              ("initializing", "start", False): _("Starting"),
              ("initializing", "start", True): _("Starting"),
              ("initializing", "auto", False): _("Starting"),
              ("initializing", "auto", True): _("Starting"),
              ("initialized", "stop", False): _("Paused"),
              ("initialized", "stop", True): _("Paused"),
              ("initialized", "start", False): _("Starting"),
              ("initialized", "start", True): _("Starting"),
              ("initialized", "auto", False): _("Queued"),
              ("initialized", "auto", True): _("Complete"),
              ("running", "stop", False): _("Downloading"),
              ("running", "stop", True): _("Seeding"),
              ("running", "start", False): _("Downloading"),
              ("running", "start", True): _("Seeding"),
              ("running", "auto", False): _("Downloading"),
              ("running", "auto", True): _("Complete"),
              ("finishing", "stop", False): _("Finishing"),
              ("finishing", "stop", True): _("Finishing"),
              ("finishing", "start", False): _("Finishing"),
              ("finishing", "start", True): _("Finishing"),
              ("finishing", "auto", False): _("Finishing"),
              ("finishing", "auto", True): _("Finishing"),
              ("failed", "stop", False): _("Error"),
              ("failed", "stop", True): _("Error"),
              ("failed", "start", False): _("Error"),
              ("failed", "start", True): _("Error"),
              ("failed", "auto", False): _("Error"),
              ("failed", "auto", True): _("Error"),}


def percentify(fraction, completed):
    if fraction is None:
        return None

    if completed and fraction >= 1.0:
        percent = 100.0
    else:
        percent = min(99.9, math.floor(fraction * 1000.0) / 10.0)

    return percent


class Size(long):
    """displays size in human-readable format"""

    size_labels = ['','K','M','G','T','P','E','Z','Y']
    radix = 2**10

    def __new__(cls, value=None, precision=None):
        if value is None:
            self = long.__new__(cls, 0)
            self.empty = True
        else:
            self = long.__new__(cls, value)
            self.empty = False
        return self

    def __init__(self, value, precision=0):
        long.__init__(self, value)
        self.precision = precision

    def __str__(self, precision=None):
        if self.empty:
            return ''
        if precision is None:
            precision = self.precision
        value = self
        for unitname in self.size_labels:
            if value < self.radix and precision < self.radix:
                break
            value /= self.radix
            precision /= self.radix
        if unitname and value < 10 and precision < 1:
            return '%.1f %sB' % (value, unitname)
        else:
            return '%.0f %sB' % (value, unitname)


class Rate(Size):
    """displays rate in human-readable format"""

    def __init__(self, value=None, precision=2**10):
        Size.__init__(self, value, precision)

    def __str__(self, precision=2**10):
        if self.empty:
            return ''
        return '%s/s'% Size.__str__(self, precision=precision)


class Duration(float):
    """displays duration in human-readable format"""

    def __new__(cls, value=None):
        if value == None:
            self = float.__new__(cls, 0)
            self.empty = True
        else:
            self = float.__new__(cls, value)
            self.empty = False
        return self

    def __str__(self):
        if self.empty or self > 365 * 24 * 60 * 60:
            return ''
        elif self >= 172800:
            return _("%d days") % (self//86400) # 2 days or longer
        elif self >= 86400:
            return _("1 day %d hours") % ((self-86400)//3600) # 1-2 days
        elif self >= 3600:
            return _("%d:%02d hours") % (self//3600, (self%3600)//60) # 1 h - 1 day
        elif self >= 60:
            return _("%d:%02d minutes") % (self//60, self%60) # 1 minute to 1 hour
        elif self >= 0:
            return _("%d seconds") % int(self)
        else:
            return _("0 seconds")


def ip_sort(a_str,b_str):
    """Fast IP address sorting function"""
    for a,b in itertools.izip(a_str.split('.'), b_str.split('.')):
        if a == b:
            continue
        if len(a) == len(b):
            return cmp(a,b)
        return cmp(int(a), int(b))
    return 0


def find_dir(path):
    if os.path.isdir(path):
        return path
    directory, garbage = os.path.split(path)
    while directory:
        if os.access(directory, os.F_OK) and os.access(directory, os.W_OK):
            return directory
        directory, garbage = os.path.split(directory)
        if garbage == '':
            break
    return None

def smart_dir(path):
    path = find_dir(path)
    if path is None:
        path = desktop
    return path

if os.name == 'nt':
    disk_term = _("drive")
elif os.name == 'posix' and os.uname()[0] == 'Darwin':
    disk_term = _("volume")
else:
    disk_term = _("disk")


class BasicTorrentObject(object):
    """Object for holding all information about a torrent"""

    def __init__(self, torrent):
        self.torrent = torrent
        self.pending = None
        self.infohash   = torrent.metainfo.infohash
        self.metainfo   = torrent.metainfo
        self.destination_path = torrent.destination_path
        self.working_path     = torrent.working_path

        self.state = torrent.state
        self.policy = torrent.policy
        self.completed = torrent.completed
        self.priority = torrent.priority
        self.completion = None
        self.piece_states = None

        self.uptotal    = 0
        self.downtotal  = 0
        self.up_down_ratio = 0
        self.peers = 0

        self.statistics = {}

        self.handler = logging.handlers.MemoryHandler(0) # capacity is ignored
        logging.getLogger("core.MultiTorrent." + repr(self.infohash)).addHandler(self.handler)


    def update(self, torrent, statistics):
        self.torrent = torrent
        self.statistics = statistics

        self.destination_path = torrent.destination_path
        self.working_path = torrent.working_path

        self.state = torrent.state
        self.policy = torrent.policy
        self.completed = torrent.completed

        self.priority = statistics['priority']
        self.completion = statistics['fractionDone']
        self.piece_states = statistics['pieceStates']

        self.uptotal   += statistics.get('upTotal'  , 0)
        self.downtotal += statistics.get('downTotal', 0)
        try:
            self.up_down_ratio = self.uptotal / self.torrent.metainfo.total_bytes
        except ZeroDivisionError:
            self.up_down_ratio = 0

        self.peers = statistics.get('numPeers', 0)


    def wants_peers(self):
        return True


    def wants_files(self):
        return self.metainfo.is_batch


    def clean_up(self):
        logging.getLogger("core.MultiTorrent." + repr(self.infohash)).removeHandler(self.handler)



class BasicApp(object):
    torrent_object_class = BasicTorrentObject

    def __init__(self, config):
        self.started = 0
        self.multitorrent = None
        self.config = config
        self.torrents = {}
        self.external_torrents = []
        self.installer_to_launch_at_exit = None
        self.logger = logging.getLogger('UI')

        self.next_autoupdate_nag = bttime()

        def gui_wrap(_f, *args, **kwargs):
            f(*args, **kwargs)

        self.gui_wrap = gui_wrap
        self.open_external_torrents_deferred = None


    def quit(self):
        if self.doneflag:
            self.doneflag.set()


    def visit_url(self, url, callback=None):
        """Visit a URL in the user's browser"""
        t = threading.Thread(target=self._visit_url, args=(url, callback))
        t.start()


    def _visit_url(self, url, callback=None):
        """Visit a URL in the user's browser non-blockingly"""
        webbrowser.open(url)
        if callback:
            self.gui_wrap(callback)


    def open_torrent_arg(self, path):
        """Open a torrent from path (URL, file) non-blockingly"""
        df = ThreadedDeferred(self.gui_wrap, GetTorrent.get_quietly, path)
        return df


    def publish_torrent(self, torrent, publish_path):
        df = self.open_torrent_arg(torrent)
        yield df
        try:
            metainfo = df.getResult()
        except GetTorrent.GetTorrentException:
            self.logger.exception("publish_torrent failed")
            return
        df = self.multitorrent.create_torrent(metainfo, publish_path, publish_path)
        yield df
        df.getResult()


    def open_torrent_arg_with_callbacks(self, path):
        """Open a torrent from path (URL, file) non-blockingly, and
        call the appropriate GUI callback when necessary."""
        def errback(e):
            exc_type, value, tb = e
            if issubclass(exc_type, GetTorrent.GetTorrentException):
                self.logger.critical(unicode(value.args[0]))
            else:
                self.logger.error("open_torrent_arg_with_callbacks failed", exc_info=e)
        def callback(metainfo):
            def open(metainfo):
                df = self.multitorrent.torrent_known(metainfo.infohash)
                yield df
                known = df.getResult()
                if known:
                    self.torrent_already_open(metainfo)
                else:
                    df = self.open_torrent_metainfo(metainfo)
                    if df is not None:
                        yield df
                        try:
                            df.getResult()
                        except TorrentAlreadyInQueue:
                            pass
                        except TorrentAlreadyRunning:
                            pass
            launch_coroutine(self.gui_wrap, open, metainfo)
        df = self.open_torrent_arg(path)
        df.addCallback(callback)
        df.addErrback(errback)
        return df


    def append_external_torrents(self, *a):
        """Append external torrents (such as those specified on the
        command line) so that they can be processed (for save paths,
        error reporting, etc.) once the GUI has started up."""
        self.external_torrents.extend(a)


    def _open_external_torrents(self):
        """Open torrents added externally (on the command line before
        startup) in a non-blocking yet serial way."""
        while self.external_torrents:
            arg = self.external_torrents.pop(0)
            df = self.open_torrent_arg(arg)
            yield df

            try:
                metainfo = df.getResult()
            except GetTorrent.GetTorrentException:
                self.logger.exception("Failed to get torrent")
                continue

            if metainfo is not None:
                # metainfo may be none if IE passes us a path to a
                # file in its cache that has already been deleted
                # because it came from a website which set
                # Pragma:No-Cache on it.
                # See GetTorrent.get_quietly().

                df = self.multitorrent.torrent_known(metainfo.infohash)
                yield df
                known = df.getResult()
                if known:
                    self.torrent_already_open(metainfo)
                else:
                    df = self.open_torrent_metainfo(metainfo)
                    if df is not None:
                        yield df
                        try:
                            df.getResult()
                        except TorrentAlreadyInQueue:
                            pass
                        except TorrentAlreadyRunning:
                            pass
        self.open_external_torrents_deferred = None


    def open_external_torrents(self):
        """Open torrents added externally (on the command line before startup)."""
        if self.open_external_torrents_deferred is None and \
               len(self.external_torrents):
            self.open_external_torrents_deferred = launch_coroutine(self.gui_wrap, self._open_external_torrents)
            def callback(*a):
                self.open_external_torrents_deferred = None
            def errback(e):
                callback()
                self.logger.error("open_external_torrents failed:", exc_info=e)

            self.open_external_torrents_deferred.addCallback(callback)
            self.open_external_torrents_deferred.addErrback(errback)


    def torrent_already_open(self, metainfo):
        """Tell the user."""
        raise NotImplementedError('BasicApp.torrent_already_open() not implemented')


    def open_torrent_metainfo(self, metainfo):
        """Get a valid save path from the user, and then tell
        multitorrent to create a new torrent from metainfo."""
        raise NotImplementedError('BasicApp.open_torrent_metainfo() not implemented')


    def launch_torrent(self, infohash):
        """Launch the torrent contents according to operating system."""
        if self.torrents.has_key(infohash):
            torrent = self.torrents[infohash]
            if torrent.metainfo.is_batch:
                LaunchPath.launchdir(torrent.working_path)
            else:
                LaunchPath.launchfile(torrent.working_path)


    def launch_torrent_folder(self, infohash):
        """Launch the torrent location according to operating system."""
        if self.torrents.has_key(infohash):
            torrent = self.torrents[infohash]
            if torrent.metainfo.is_batch:
                LaunchPath.launchdir(torrent.working_path)
            else:
                path, file = os.path.split(torrent.working_path)
                LaunchPath.launchdir(path)


    def launch_installer_at_exit(self):
        LaunchPath.launchfile(self.installer_to_launch_at_exit)


    def do_log(self, severity, text):
        raise NotImplementedError('BasicApp.do_log() not implemented')

    def init_updates(self):
        """Make status request at regular intervals."""
        raise NotImplementedError('BasicApp.init_updates() not implemented')


    def make_statusrequest(self, event = None):
        """Make status request."""
        df = launch_coroutine(self.gui_wrap, self.update_status)
        def errback(e):
            self.logger.error("update_status failed", exc_info=e)
        df.addErrback(errback)
        return True


    def update_single_torrent(self, infohash):
        torrent = self.torrents[infohash]
        df = self.multitorrent.torrent_status(infohash,
                                              torrent.wants_peers(),
                                              torrent.wants_files()
                                              )
        yield df
        try:
            core_torrent, statistics = df.getResult()
        except UnknownInfohash:
            # looks like it's gone now
            if infohash in self.torrents:
                self.torrents.pop(infohash)
                self.torrent_removed(infohash)
        else:
            # the infohash might have been removed from torrents
            # while we were yielding above, so we need to check
            if infohash in self.torrents:
                core_torrent = ThreadProxy(core_torrent, self.gui_wrap)
                torrent.update(core_torrent, statistics)
                self.update_torrent(torrent)


    def update_status(self):
        """Update torrent information based on the results of making a
        status request."""
        df = self.multitorrent.get_torrents()
        yield df
        torrents = df.getResult()

        infohashes = set()
        au_torrents = {}

        for torrent in torrents:
            torrent = ThreadProxy(torrent, self.gui_wrap)
            infohashes.add(torrent.metainfo.infohash)
            if (not torrent.hidden and
                torrent.metainfo.infohash not in self.torrents):
                # create new torrent widget
                to = self.new_displayed_torrent(torrent)

            if torrent.is_auto_update:
                au_torrents[torrent.metainfo.infohash] = torrent

        for infohash, torrent in copy(self.torrents).iteritems():
            # remove nonexistent torrents
            if infohash not in infohashes:
                self.torrents.pop(infohash)
                self.torrent_removed(infohash)

        total_completion = 0
        total_bytes = 0

        for infohash, torrent in copy(self.torrents).iteritems():
            # update existing torrents
            df = self.multitorrent.torrent_status(infohash,
                                                  torrent.wants_peers(),
                                                  torrent.wants_files()
                                                  )
            yield df
            try:
                core_torrent, statistics = df.getResult()
            except UnknownInfohash:
                # looks like it's gone now
                if infohash in self.torrents:
                    self.torrents.pop(infohash)
                    self.torrent_removed(infohash)
            else:
                # the infohash might have been removed from torrents
                # while we were yielding above, so we need to check
                if infohash in self.torrents:
                    core_torrent = ThreadProxy(core_torrent, self.gui_wrap)
                    torrent.update(core_torrent, statistics)
                    self.update_torrent(torrent)
                    if statistics['fractionDone'] is not None:
                        amount_done = statistics['fractionDone'] * torrent.metainfo.total_bytes
                        total_completion += amount_done
                        total_bytes += torrent.metainfo.total_bytes
        all_completed = False
        if total_bytes == 0:
            average_completion = 0
        else:
            average_completion = total_completion / total_bytes
            if total_completion == total_bytes:
                all_completed = True

        df = self.multitorrent.auto_update_status()
        yield df
        available_version, installable_version, delay = df.getResult()
        if available_version is not None:
            if installable_version is None:
                self.notify_of_new_version(available_version)
            else:
                if self.installer_to_launch_at_exit is None:
                    atexit.register(self.launch_installer_at_exit)
                torrent = au_torrents[installable_version]
                self.installer_to_launch_at_exit = torrent.working_path
                if bttime() > self.next_autoupdate_nag:
                    self.prompt_for_quit_for_new_version(available_version)
                    self.next_autoupdate_nag = bttime() + delay

        def get_global_stats(mt):
            stats = {}

            u, d = mt.get_total_rates()
            stats['total_uprate'] = Rate(u)
            stats['total_downrate'] = Rate(d)

            u, d = mt.get_total_totals()
            stats['total_uptotal'] = Size(u)
            stats['total_downtotal'] = Size(d)

            torrents = mt.get_visible_torrents()
            running = mt.get_visible_running()
            stats['num_torrents'] = len(torrents)
            stats['num_running_torrents'] = len(running)

            stats['num_connections'] = 0
            for t in torrents:
                stats['num_connections'] += t.get_num_connections()

            try:
                stats['avg_connections'] = (stats['num_connections'] /
                                            stats['num_running_torrents'])
            except ZeroDivisionError:
                stats['avg_connections'] = 0

            stats['avg_connections'] = "%.02f" % stats['avg_connections']

            return stats

        df = self.multitorrent.call_with_obj(get_global_stats)
        yield df
        global_stats = df.getResult()

        yield average_completion, all_completed, global_stats


    def _update_status(self, total_completion):
        raise NotImplementedError('BasicApp._update_status() not implemented')


    def new_displayed_torrent(self, torrent):
        """Tell the UI that it should draw a new torrent."""
        torrent_object = self.torrent_object_class(torrent)
        self.torrents[torrent.metainfo.infohash] = torrent_object
        return torrent_object

    def torrent_removed(self, infohash):
        """Tell the GUI that a torrent has been removed, by it, or by
        multitorrent."""
        raise NotImplementedError('BasicApp.torrent_removed() removing missing torrents not implemented')


    def update_torrent(self, torrent_object):
        """Tell the GUI to update a torrent's info."""
        raise NotImplementedError('BasicApp.update_torrent() updating existing torrents not implemented')

    def notify_of_new_version(self, version):
        print 'got auto_update_status', version
        pass

    def prompt_for_quit_for_new_version(self, version):
        print 'got new version', version
        pass

    # methods that are used to send commands to MultiTorrent
    def send_config(self, option, value, infohash=None):
        """Tell multitorrent to set a config item."""
        self.config[option] = value
        if self.multitorrent:
            self.multitorrent.set_option(option, value, infohash)


    def remove_infohash(self, infohash, del_files=False):
        """Tell multitorrent to remove a torrent."""
        df = self.multitorrent.remove_torrent(infohash, del_files=del_files)
        yield df
        try:
            df.getResult()
        except KeyError:
            pass # it was already gone, who cares

        if infohash in self.torrents:
            self.torrent_removed(infohash)
            tw = self.torrents.pop(infohash)
            tw.clean_up()


    def set_file_priority(self, infohash, filenames, dowhat):
        """Tell multitorrent to set file priorities."""
        for f in filenames:
            self.multitorrent.set_file_priority(infohash, f, dowhat)


    def stop_torrent(self, infohash, pause=False):
        """Tell multitorrent to stop a torrent."""
        torrent = self.torrents[infohash]
        if (torrent and torrent.pending == None):
            torrent.pending = "stop"

            if not pause:
                df = self.multitorrent.set_torrent_policy(infohash, "stop")
                yield df
                try:
                    df.getResult()
                except TorrentNotRunning:
                    pass

            if torrent.state == "running":
                df = self.multitorrent.stop_torrent(infohash, pause=pause)
                yield df
                torrent.state = df.getResult()

            torrent.pending = None
            yield True


    def start_torrent(self, infohash):
        """Tell multitorrent to start a torrent."""
        torrent = self.torrents[infohash]
        if (torrent and torrent.pending == None and
            torrent.state in ["failed", "initialized"]):
            torrent.pending = "start"

            if torrent.state == "failed":
                df = self.multitorrent.reinitialize_torrent(infohash)
                yield df
                df.getResult()

            df = self.multitorrent.set_torrent_policy(infohash, "auto")
            yield df
            df.getResult()
                
            torrent.pending = None
            yield True


    def force_start_torrent(self, infohash):
        torrent = self.torrents[infohash]
        if (torrent and torrent.pending == None):
            torrent.pending = "force start"

            df = self.multitorrent.set_torrent_policy(infohash, "start")
            yield df
            df.getResult()

            if torrent.state in ["failed", "initialized"]:
                if torrent.state == "failed":
                    df = self.multitorrent.reinitialize_torrent(infohash)
                    yield df
                    df.getResult()

                df = self.multitorrent.start_torrent(infohash)
                yield df
                try:
                    torrent.state = df.getResult()
                except TorrentAlreadyRunning:
                    torrent.state = "running"

            torrent.pending = None
            yield True


    def external_command(self, action, *datas):
        """For communication via IPC"""
        if action == 'start_torrent':
            assert len(datas) == 1
            self.append_external_torrents(*datas)
            self.logger.info('got external_command:start_torrent: "%s"' % datas[0])
            # this call does Ye Olde Threadede Deferrede:
            self.open_external_torrents()
        elif action == 'publish_torrent':
            self.logger.info('got external_command:publish_torrent: "%s" as "%s"' % datas)
            launch_coroutine(self.gui_wrap, self.publish_torrent, datas[0], datas[1])
        elif action == 'show_error':
            assert len(datas) == 1
            self.logger.error(datas[0])
        elif action == 'no-op':
            self.logger.info('got external_command: no-op')
        else:
            self.logger.warning('got unknown external_command: %s' % str(action))
            # fun.
            #code = action + ' '.join(datas)
            #self.logger.warning('eval: %s' % code)
            #exec code



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# Written by Matt Chisholm

import os
import wx

from BitTorrent.StatusLight import StatusLight as _StatusLight

class StatusLight(wx.Panel, _StatusLight):

    def __init__(self, parent, wxid=wx.ID_ANY):
        images = {}
        tips = {}
        a = wx.GetApp()
        for k in self.states.keys():
            i = a.theme_library.get(('statuslight', self.states[k][0]))
            b = wx.BitmapFromImage(i)
            images[k] = b

        _StatusLight.__init__(self)
        wx.Panel.__init__(self, parent, id=wxid, style=wx.NO_BORDER)
        self.SetSize(wx.Size(24,24))
        self.bitmap = wx.StaticBitmap(self, wx.ID_ANY)

        self.images = images
        self.tips = tips

        self.Fit()

        self.change_state()


    def change_state(self):
        state = self.mystate
        assert self.states.has_key(state)

        self.bitmap.SetBitmap(self.images[state])
        self.SetToolTipString(self.get_tip())
        
class StatusLabel(_StatusLight):
    pass    



# The contents of this file are subject to the BitTorrent Open Source License
# Version 1.1 (the License).  You may not copy or use this file, in either
# source code or executable form, except in compliance with the License.  You
# may obtain a copy of the License at http://www.bittorrent.com/license/.
#
# Software distributed under the License is distributed on an AS IS basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
# for the specific language governing rights and limitations under the
# License.

# written by Matt Chisholm

import os
import sys
import pickle
import zurllib
import BitTorrent.stackthreading as threading
import logging
from BitTorrent.hash import sha
from BitTorrent.translation import _

DEBUG = False

from BitTorrent import BTFailure, version, app_name
from BitTorrent import GetTorrent
from BitTorrent.bencode import bdecode, bencode
from BitTorrent.platform import os_version, spawn, get_temp_dir, doc_root, is_frozen_exe, osx
from BitTorrent.ConvertedMetainfo import ConvertedMetainfo


if osx:
    from Foundation import NSAutoreleasePool
    
if is_frozen_exe or DEBUG:
    # needed for py2exe to include the public key lib
    from Crypto.PublicKey import DSA

version_host = 'http://version.bittorrent.com/'
download_url = 'http://bittorrent.com/download.html'

# based on Version() class from ShellTools package by Matt Chisholm,
# used with permission
class Version(list):
    def __str__(self):
        return '.'.join(map(str, self))

    def is_beta(self):
        return self[1] % 2 == 1

    def from_str(self, text):
        return Version( [int(t) for t in text.split('.')] )

    def name(self):
        if self.is_beta():
            return 'beta'
        else:
            return 'stable'
    
    from_str = classmethod(from_str)

currentversion = Version.from_str(version)

availableversion = None

class Updater(object):
    def __init__(self, threadwrap, newversionfunc, startfunc, installfunc,
                 errorfunc, test_new_version='', test_current_version=''):
        self.threadwrap     = threadwrap  # for calling back to UI from thread
        self.newversionfunc = newversionfunc # alert to new version UI function
        self.startfunc      = startfunc   # start torrent UI function
        self.installfunc    = installfunc # install torrent UI function
        self.errorfunc      = errorfunc   # report error UI function
        self.infohash = None
        self.version = currentversion
        self.currentversion = currentversion
        self.asked_for_install = False
        self.version_site = version_host
        if os.name == 'nt':
            self.version_site += 'win32/'
            if os_version not in ('XP', '2000', '2003'):
                self.version_site += 'legacy/'
        elif osx:
            self.version_site += 'osx/'
        self.debug_mode = DEBUG
        if test_new_version:
            test_new_version = Version.from_str(test_new_version)
            self.debug_mode = True
            def _hack_get_available(url):
                return test_new_version
            self._get_available = _hack_get_available
        if test_current_version:
            self.debug_mode = True
            self.currentversion = Version.from_str(test_current_version)


    def debug(self, message):
        if self.debug_mode:
            self.threadwrap(self.errorfunc, logging.WARNING, message)


    def _get_available(self, url):
        self.debug('Updater.get_available() hitting url %s' % url)
        try:
            u = zurllib.urlopen(url)
            s = u.read()
            s = s.strip()
        except:
            raise BTFailure(_("Could not get latest version from %s")%url)
        try:
            assert len(s) == 5
            availableversion = Version.from_str(s)
        except:
            raise BTFailure(_("Could not parse new version string from %s")%url)        
        return availableversion


    def get_available(self):
        url = self.version_site + self.currentversion.name()
        availableversion = self._get_available(url)
        if availableversion.is_beta():
            if availableversion[1] != self.currentversion[1]:
                availableversion = self.currentversion 
        if self.currentversion.is_beta():
            stable_url = self.version_site + 'stable'
            available_stable_version = self._get_available(stable_url)
            if available_stable_version > availableversion:
                availableversion = available_stable_version
        self.version = availableversion
        self.debug('Updater.get_available() got %s' % str(self.version))
        return self.version


    def get(self):
        try:
            self.get_available()
        except BTFailure, e:
            self.threadwrap(self.errorfunc, logging.WARNING, e)
            return 

        if self.version <= self.currentversion:
            self.debug('Updater.get() not updating old version %s' % str(self.version))
            return

        if not self.can_install():
            self.debug('Updater.get() cannot install on this os')
            return

        self.installer_name = self.calc_installer_name()
        self.installer_url  = self.version_site + self.installer_name + '.torrent'
        self.installer_dir  = self.calc_installer_dir()

        self.torrentfile = None
        try:
            self.torrentfile = GetTorrent.get_url(self.installer_url)
        except GetTorrent.GetTorrentException, e:
            terrors = [unicode(e.args[0])]

        signature = None
        try:
            signfile = zurllib.urlopen(self.installer_url + '.sign')
        except:
            self.debug('Updater.get() failed to get signfile %s.sign' % self.installer_url)
        else:
            try:
                signature = pickle.load(signfile)
            except:
                self.debug('Updater.get() failed to load signfile %s' % signfile)
        
        if terrors:
            self.threadwrap(self.errorfunc, logging.WARNING, '\n'.join(terrors))

        if torrentfile and signature:
            public_key_file = open(os.path.join(doc_root, 'public.key'), 'rb')
            public_key = pickle.load(public_key_file)
            h = sha(torrentfile).digest()
            if public_key.verify(h, signature):
                self.torrentfile = torrentfile
                b = bdecode(torrentfile)
                self.infohash = sha(bencode(b['info'])).digest()
                self.total_size = b['info']['length']
                self.debug('Updater.get() got torrent file and signature')
            else:
                self.debug('Updater.get() torrent file signature failed to verify.')
                pass
        else:
            self.debug('Updater.get() doesn\'t have torrentfile %s and signature %s' %
                       (str(type(torrentfile)), str(type(signature))))

    def installer_path(self):
        if self.installer_dir is not None:
            return os.path.join(self.installer_dir,
                                self.installer_name)
        else:
            return None
        
    def check(self):
        t = threading.Thread(target=self._check,
                             args=())
        t.setDaemon(True)
        t.start()

    def _check(self):
        if osx:
            pool = NSAutoreleasePool.alloc().init()
        self.get()
        if self.version > self.currentversion:
            self.threadwrap(self.newversionfunc, self.version, download_url)

    def can_install(self):
        if self.debug_mode:
            return True
        if os.name == 'nt':
            return True
        elif osx:
            return True
        else:
            return False

    def calc_installer_name(self):
        if os.name == 'nt':
            ext = 'exe'
        elif osx:
            ext = 'dmg'
        elif os.name == 'posix' and self.debug_mode: 
            ext = 'tar.gz' 
        else:
            return
        
        parts = [app_name, str(self.version)]
        if self.version.is_beta():
            parts.append('Beta')
        name = '-'.join(parts)
        name += '.' + ext
        return name

    def set_installer_dir(self, path):
        self.installer_dir = path
        
    def calc_installer_dir(self):
        if hasattr(self, 'installer_dir'):
            return self.installer_dir
        
        temp_dir = get_temp_dir()
        if temp_dir is not None:
            return temp_dir
        else:
            self.errorfunc(logging.WARNING,
                           _("Could not find a suitable temporary location to "
                             "save the %s %s installer.") % (app_name, self.version))

    def installer_downloaded(self):
        if self.installer_path() and os.access(self.installer_path(), os.F_OK):
            size = os.stat(self.installer_path())[6]
            if size == self.total_size:
                return True
            else:
                #print 'installer is wrong size, is', size, 'should be', self.total_size
                return False
        else:
            #print 'installer does not exist'
            return False

    def download(self):
        if self.torrentfile is not None:
            self.startfunc(self.torrentfile, self.installer_path())
        else:
            self.errorfunc(logging.WARNING, _("No torrent file available for %s %s "
                                      "installer.")%(app_name, self.version))

    def start_install(self):
        if not self.asked_for_install:
            if self.installer_downloaded():
                self.asked_for_install = True
                self.installfunc()
            else:
                self.errorfunc(logging.WARNING,
                               _("%s %s installer appears to be incomplete, "
                                 "missing, or corrupt.")%(app_name,
                                                          self.version))

    def launch_installer(self, torrentqueue):
        if os.name == 'nt':
            spawn(torrentqueue, self.installer_path(), "/S")
        else:
            self.errorfunc(logging.WARNING, _("Cannot launch installer on this OS"))
